#include "bezierCurves.h"


BezierCurve::BezierCurve()
{
    m_firstControlPoints.clear();
    m_secondControlPoints.clear();
}


QPointF BezierCurve::getFirstControlPoint(int index)
{
    return m_firstControlPoints[index];
}
QPointF BezierCurve::getSecondControlPoint(int index)
{
    return m_secondControlPoints[index];
}



int BezierCurve::calcCurveControlPoints(CurvePointsList knots)
{
    int n = knots.size() - 1;
    if (n < 1)
    {
        qDebug() << "At least two knot points required";
        return 1;
    }

    if (n == 1)
    { // Special case: Bezier curve should be a straight line.
        m_firstControlPoints.resize(1);
        // 3P1 = 2P0 + P3
        m_firstControlPoints[0].setX( (2 * knots[0].x() + knots[1].x()) / 3 );
        m_firstControlPoints[0].setY( (2 * knots[0].y() + knots[1].y()) / 3 );

        m_secondControlPoints.resize(1);
        // P2 = 2P1 – P0
        m_secondControlPoints[0].setX( 2*m_firstControlPoints[0].x() - knots[0].x() );
        m_secondControlPoints[0].setY( 2*m_firstControlPoints[0].y() - knots[0].y() );
        return 0;
    }

    // Calculate first Bezier control points
    // Right hand side vector
    QVector<double> rhs = QVector<double> (n);

    // Set right hand side X values
    for (int i = 1; i < n - 1; ++i)
        rhs[i] = 4 * knots[i].x() + 2 * knots[i + 1].x();
    rhs[0] = knots[0].x() + 2 * knots[1].x();
    rhs[n - 1] = (8 * knots[n - 1].x() + knots[n].x()) / 2.0;
    // Get first control points X-values
    QVector<double> x = calcFirstControlPoints(rhs);

    // Set right hand side Y values
    for (int i = 1; i < n - 1; ++i)
        rhs[i] = 4 * knots[i].y() + 2 * knots[i + 1].y();
    rhs[0] = knots[0].y() + 2 * knots[1].y();
    rhs[n - 1] = (8 * knots[n - 1].y() + knots[n].y()) / 2.0;
    // Get first control points Y-values
    QVector<double> y = calcFirstControlPoints(rhs);

    // Fill output arrays.
    m_firstControlPoints.resize(n);
    m_secondControlPoints.resize(n);
    for (int i = 0; i < n; ++i)
    {
        // First control point
        m_firstControlPoints[i] = QPointF(x[i], y[i]);
        // Second control point
        if (i < n - 1)
            m_secondControlPoints[i] = QPointF(2 * knots
                [i + 1].x() - x[i + 1], 2 *
                knots[i + 1].y() - y[i + 1]);
        else
            m_secondControlPoints[i] = QPointF((knots
                [n].x() + x[n - 1]) / 2,
                (knots[n].y() + y[n - 1]) / 2);
    }
    return 0;
}


QVector<double> BezierCurve::calcFirstControlPoints(QVector<double> rhs)
{
    int n = rhs.size();
    QVector<double> x(n); // Solution vector.
    QVector<double> tmp(n); // Temp workspace.

    double b = 2.0;
    x[0] = rhs[0] / b;
    for (int i = 1; i < n; i++) // Decomposition and forward substitution.
    {
        tmp[i] = 1 / b;
        b = (i < n - 1 ? 4.0 : 3.5) - tmp[i];
        x[i] = (rhs[i] - x[i - 1]) / b;
    }
    for (int i = 1; i < n; i++)
        x[n - i - 1] -= tmp[n - i] * x[n - i]; // Backsubstitution.

    return x;
}

