#include "circuit.h"



Circuit::Circuit(double _frequencyGHz)
{
    // Only used to add new component with suited default value
    m_defaultFrequencyGHz = _frequencyGHz;

    m_componentsList.clear();

    // Pixmap vector initialization
    for (int i=0 ; i<NB_COMP_TYPE-1 ; i++)
        pixmapComponents[i] = new QPixmap();

    pixmapSuppComp = new QPixmap();
    pixmapDotAndPoint = new QPixmap();

    // Pixmap components generation
    generatePixmapsComponents(64,64);
    qDebug() << "Comp pixmaps loaded : OK";
}



void Circuit::addComponent(Component comp, bool componentPreview)
{
    int last=-1;
    // preview : comp added at the end
    if (componentPreview==true)
    {
        m_componentsList.append(comp);
        m_componentsList.last().setPreviewState(componentPreview);
        qDebug() << "Component preview added :" << m_componentsList.last().getName();
    }
    else // not preview, added after the last not preview comp
    {
        if (m_componentsList.size()==0)
            last=-1;
        else
        {
            // find last not preview comp
            for (int i=0 ; i<m_componentsList.size() ; i++)
            {
                if (m_componentsList[i].getPreviewState()==false)
                    last=i;
            }
        }
        m_componentsList.insert(last+1, comp);
        m_componentsList[last+1].setPreviewState(componentPreview);
        qDebug() << "Component added :" << m_componentsList[last+1].getName();
    }
}

void Circuit::addComponent(ComponentType _compType, CompParametersList _parameters, bool _componentPreview)
{
    Component comp(_compType, _parameters);
    addComponent(comp, _componentPreview);
}


void Circuit::addDefaultComponent(ComponentType type, bool componentPreview)
{
    switch (type)
    {
    case CAPA_S:
        addNewCAPA_S(componentPreview); break;
    case SELF_S:
        addNewSELF_S(componentPreview); break;
    case RESIST_S:
        addNewRESIST_S(componentPreview); break;
    case LINE_S:
        addNewLINE_S(componentPreview); break;
    case CAPA_P:
        addNewCAPA_P(componentPreview); break;
    case SELF_P:
        addNewSELF_P(componentPreview); break;
    case RESIST_P:
        addNewRESIST_P(componentPreview); break;
    case LINE_OPEN:
        addNewLINE_OPEN(componentPreview); break;
    case LINE_SHORT:
        addNewLINE_SHORT(componentPreview); break;
    case NOT_DEF:
        break;
    }
}

// CAPA_S, SELF_S, RESIST_S, LINE_S, CAPA_P, SELF_P, RESIST_P, LINE_OPEN, LINE_SHORT, NOT_DEF };
void Circuit::insertCAPA_S(int numCompBefore) {
    m_componentsList.insert(numCompBefore, Component(CAPA_S, 400.0/m_defaultFrequencyGHz));
}
void Circuit::insertSELF_S(int numCompBefore) {
    m_componentsList.insert(numCompBefore, Component(SELF_S, 0.08/m_defaultFrequencyGHz));
}
void Circuit::insertRESIST_S(int numCompBefore) {
    m_componentsList.insert(numCompBefore, Component(RESIST_S, 0.3));
}
void Circuit::insertLINE_S(int numCompBefore)
{
    CompParametersList paramList;
    paramList.append(24.0/m_defaultFrequencyGHz); // 24mm mechanical length by default at 1GHz
    paramList.append(1.0); // 50ohm normalized
    paramList.append(1.0); // Er
    m_componentsList.insert(numCompBefore, Component(LINE_S, paramList));
}
void Circuit::insertCAPA_P(int numCompBefore) {
    m_componentsList.insert(numCompBefore, Component(CAPA_P, 400.0/m_defaultFrequencyGHz));
}
void Circuit::insertSELF_P(int numCompBefore) {
    m_componentsList.insert(numCompBefore, Component(SELF_P, 0.06/m_defaultFrequencyGHz));
}
void Circuit::insertRESIST_P(int numCompBefore) {
    m_componentsList.insert(numCompBefore, Component(RESIST_P, 0.2));
}
void Circuit::insertLINE_OPEN(int numCompBefore) {
    CompParametersList paramList;
    paramList.append(60.0/m_defaultFrequencyGHz); // 24mm mechanical length by default at 1GHz
    paramList.append(1.0); // 50ohm normalized
    paramList.append(1.0); // Er
    m_componentsList.insert(numCompBefore, Component(LINE_OPEN, paramList));
}
void Circuit::insertLINE_SHORT(int numCompBefore) {
    CompParametersList paramList;
    paramList.append(20.0/m_defaultFrequencyGHz); // 24mm mechanical length by default at 1GHz
    paramList.append(1.0); // 50ohm normalized
    paramList.append(1.0); // Er
    m_componentsList.insert(numCompBefore, Component(LINE_SHORT, paramList));
}



void Circuit::removeComponent(int numComp)
{
    if (m_componentsList.size() >= numComp)
        m_componentsList.remove(numComp);
}


void Circuit::removeLastComponent()
{
    int compToDelete=-1;

    if (!m_componentsList.isEmpty())
    {
        // find last not preview comp
        for (int i=0 ; i<m_componentsList.size() ; i++)
        {
            if (m_componentsList[i].getPreviewState()==false)
                compToDelete=i;
        }
    }
    if (compToDelete!=-1)
        m_componentsList.remove(compToDelete);
}


void Circuit::replaceComponent(int numComp, Component comp)
{
    m_componentsList.replace(numComp, comp);
}



void Circuit::setPreviewComponent(ComponentType type)
{
    removePreviewComponent();
    if (type!=NOT_DEF)
    {
        addDefaultComponent(type, true);
    }
}

void Circuit::removePreviewComponent()
{
    if (!m_componentsList.isEmpty())
    {
        for (int i=0 ; i<m_componentsList.size() ; i++)
        {
            if (m_componentsList.last().getPreviewState()==true)
                m_componentsList.removeLast();
        }
    }
}


ComponentType Circuit::getPreviewComponentType()
{
    if (!m_componentsList.isEmpty())
    {
        if (m_componentsList.last().getPreviewState() == true)
            return m_componentsList.last().getType();
    }
    return NOT_DEF;
}

void Circuit::clearAll()
{
    m_componentsList.clear();
}


void Circuit::addNewCAPA_S(bool componentPreview) {
    addComponent(Component(CAPA_S, 400.0/m_defaultFrequencyGHz), componentPreview); // normalized value (400 = 8pF with 50ohm at 1GHz)
}
void Circuit::addNewSELF_S(bool componentPreview) {
    addComponent(Component(SELF_S, 0.08/m_defaultFrequencyGHz), componentPreview); // normalized value
}
void Circuit::addNewRESIST_S(bool componentPreview) {
    addComponent(Component(RESIST_S, 0.3), componentPreview); // normalized value (0.3 = 15.0 with 50ohm)
}
void Circuit::addNewCAPA_P(bool componentPreview) {
    if (getSize()==1) // if first component (to reach 50ohm with 3 components...)
        addComponent(Component(CAPA_P, 250.0/m_defaultFrequencyGHz), componentPreview); // normalized value (400 = 8pF with 50ohm at 2GHz)
    else if (getSize()==3)
        addComponent(Component(CAPA_P, 100.0/m_defaultFrequencyGHz), componentPreview); // normalized value (400 = 8pF with 50ohm at 2GHz)
    else
        addComponent(Component(CAPA_P, 400.0/m_defaultFrequencyGHz), componentPreview); // normalized value (400 = 8pF with 50ohm at 2GHz)
}
void Circuit::addNewSELF_P(bool componentPreview) {
    addComponent(Component(SELF_P, 0.06/m_defaultFrequencyGHz), componentPreview); // normalized value
}
void Circuit::addNewRESIST_P(bool componentPreview) {
    addComponent(Component(RESIST_P, 0.2), componentPreview); // normalized value (0.2 = 10.0 with 50ohm)
}
void Circuit::addNewLINE_S(bool componentPreview)
{
    CompParametersList paramList;
    if (getSize()==2)
        paramList.append(30.0/m_defaultFrequencyGHz);
    else
        paramList.append(24.0/m_defaultFrequencyGHz); // 24mm mechanical length by default at 1GHz
    paramList.append(1.0); // 50ohm normalized
    paramList.append(1.0); // Er
    paramList.append(0.0); // alpha att.
    addComponent(Component(LINE_S, paramList), componentPreview);
}
void Circuit::addNewLINE_OPEN(bool componentPreview)
{
    CompParametersList paramList;
    paramList.append(60.0/m_defaultFrequencyGHz); // 60mm mechanical length by default at 1GHz
    paramList.append(1.0); // 50ohm normalized
    paramList.append(1.0); // Er
    paramList.append(0.0); // alpha att.
    addComponent(Component(LINE_OPEN, paramList), componentPreview);
}
void Circuit::addNewLINE_SHORT(bool componentPreview)
{
    CompParametersList paramList;
    paramList.append(20.0/m_defaultFrequencyGHz); // 20mm mechanical length by default at 1GHz
    paramList.append(1.0); // 50ohm normalized
    paramList.append(1.0); // Er
    paramList.append(0.0); // alpha att.
    addComponent(Component(LINE_SHORT, paramList), componentPreview);
}


Component Circuit::getComponentCopy(int numComp) const
{
    Component compTmp = m_componentsList[numComp];
    return compTmp;
}

ComponentsList Circuit::getComponentsList() const
{
    return m_componentsList;
}

int Circuit::getSize()
{
    return m_componentsList.size();
}


int Circuit::getSizeWithoutPreviewed()
{
    int count=0;
    for (int i=0 ; i<m_componentsList.size() ; i++)
    {
        if (m_componentsList[i].getPreviewState()==false)
            count++;
    }
    return count;
}

int Circuit::getTotalNumberOfPoints()
{
    int nbTotal=0;
    for (int i=0 ; i<m_componentsList.size() ; i++)
    {
        for (int j=0 ; j<m_componentsList[i].getDataPoints().size() ; j++)
        {
            nbTotal += m_componentsList[i].getDataPoints().at(j).size();
        }
    }
    return nbTotal;
}


QVector< QVector<complexF> > & Circuit::getComponentDataPoints(int numComp)
{
    QVector< QVector<complexF> > &tmp_p = m_componentsList[numComp].getDataPoints();
    return tmp_p;
}
void Circuit::addComponentDataPoints(int numComp, int numStartPoint, QVector<complexF> &pointVector)
{
    m_componentsList[numComp].addDataPoints(numStartPoint, pointVector);
}
void Circuit::clearComponentDataPoints(int numComp)
{
    m_componentsList[numComp].clearDataPoints();
}
void Circuit::clearComponentDataPoints(int numComp, int numStartPoint)
{
    m_componentsList[numComp].clearDataPoints(numStartPoint);
}
void Circuit::clearAllDataPoints()
{
    for (int i=0 ; i<m_componentsList.size() ; i++)
    {
        m_componentsList[i].clearDataPoints();
    }
}

QString Circuit::getComponentName(int numComp) const
{
    return m_componentsList[numComp].getName();
}

ComponentType Circuit::getComponentType(int numComp) const
{
    return m_componentsList[numComp].getType();
}

QString Circuit::getComponentStrUnit(int numComp) const
{
    return m_componentsList[numComp].getStrUnit();
}



double Circuit::getComponentParameter(int numComp, int numParam) const
{
    return m_componentsList[numComp].getParameter(numParam);
}
CompParametersList Circuit::getComponentParameters(int numComp) const
{
    return m_componentsList[numComp].getParameters();
}

double Circuit::getComponentParameterDenorm(int numComp, int numParam, double Z0)
{
    ComponentType type = m_componentsList[numComp].getType();
    double parameter = m_componentsList[numComp].getParameter(numParam);
    return (parameterNormToDenormConv( parameter, type, numParam, Z0));
}
CompParametersList Circuit::getComponentParametersDenorm(int numComp, double Z0)
{
    CompParametersList listDenorm;

    for (int i=0 ; i<m_componentsList[numComp].getParameters().size() ; i++)
    {
        listDenorm.append(getComponentParameterDenorm(numComp, i, Z0));
    }
    return listDenorm;
}


void Circuit::setComponentParameterDenorm(int numComp, int numParam, double param, double Z0)
{
    ComponentType type = m_componentsList[numComp].getType();
    m_componentsList[numComp].setParameter(numParam, parameterDenormToNormConv( param, type, numParam, Z0));

    qDebug() << "Parameter modification -> Comp n°" << numComp+1 << "on" << getSizeWithoutPreviewed() << ", new param (denorm) : [" << numParam << "] =" << param;
}

void Circuit::ratioOnCompFirstParameter(int numComp, double ratio)
{
    m_componentsList[numComp].ratioOnFirstParameter(ratio);
}

bool Circuit::getComponentPreviewState(int numComp) const
{
    return m_componentsList[numComp].getPreviewState();
}
void Circuit::setComponentPreviewState(int numComp, bool state)
{
    m_componentsList[numComp].setPreviewState(state);
}
bool Circuit::getComponentMouseOverState(int numComp) const
{
    return m_componentsList[numComp].getMouseOverState();
}
void Circuit::setComponentMouseOverState(int numComp, bool state)
{
    m_componentsList[numComp].setMouseOverState(state);
}
void Circuit::resetAllComponentsMouseOverState()
{
    for (int i=0 ; i<m_componentsList.size() ; i++)
    {
        m_componentsList[i].setMouseOverState(false);
    }
}

QPixmap Circuit::getPixmapComponent(ComponentType type, int _width, int _heigth)
{
    if (pixmapComponents[type]->width()!=_width) // redraw only if different sizes
        generatePixmapsComponents(_width, _heigth);

    return *pixmapComponents[type];
}

QPixmap Circuit::getPixmapSuppComp(int _width, int _heigth)
{
    if (pixmapSuppComp->width()!=_width) // redraw only if different sizes
        generatePixmapsComponents(_width, _heigth);

    return *pixmapSuppComp;
}

QPixmap Circuit::getPixmapDotAndPoint(int _width, int _heigth)
{
    if (pixmapDotAndPoint->width()!=_width) // redraw only if different sizes
        generatePixmapsComponents(_width, _heigth);

    return *pixmapDotAndPoint;
}


complexF Circuit::getLastPointNotInPreview(int numStartPoint, complexF defaultReturn)
{
    complexF result;
    bool pointFound=false;

    for (int i=0 ; i<m_componentsList.size() ; i++)
    {
        if (m_componentsList[i].getPreviewState()==false) // not in preview => take the last point
        {
            const QVector< QVector< complexF > > &resultPointsList = m_componentsList[i].getDataPoints();
            if (!resultPointsList.isEmpty())
            {
                result = resultPointsList[numStartPoint].last();
                pointFound=true;
            }
        }
    }

    if (pointFound)
        return result;
    else
        return defaultReturn;
}


int Circuit::getComponentSimPointsDensity(int numComp) const
{
    return m_componentsList[numComp].getSimPointsDensity();
}
void Circuit::setComponentSimPointsDensity(int numComp, int _density)
{
    m_componentsList[numComp].setSimPointsDensity(_density);
}
void Circuit::increaseComponentSimPointsDensity(int numComp)
{
    int density = m_componentsList[numComp].getSimPointsDensity();
    if (density==LOW_SIM_POINTS_DENSITY)
        density=MEDIUM_SIM_POINTS_DENSITY;
    else if (density==MEDIUM_SIM_POINTS_DENSITY)
        density=HIGH_SIM_POINTS_DENSITY;
    else if (density==HIGH_SIM_POINTS_DENSITY)
        density=V_HIGH_SIM_POINTS_DENSITY;
    else if (density==V_HIGH_SIM_POINTS_DENSITY)
        density=ULTRA_SIM_POINTS_DENSITY;
    m_componentsList[numComp].setSimPointsDensity(density);
}
void Circuit::decreaseComponentSimPointsDensity(int numComp)
{
    int density = m_componentsList[numComp].getSimPointsDensity();
    if (density==ULTRA_SIM_POINTS_DENSITY)
        density=V_HIGH_SIM_POINTS_DENSITY;
    else if (density==V_HIGH_SIM_POINTS_DENSITY)
        density=HIGH_SIM_POINTS_DENSITY;
    else if (density==HIGH_SIM_POINTS_DENSITY)
        density=MEDIUM_SIM_POINTS_DENSITY;
    else if (density==MEDIUM_SIM_POINTS_DENSITY)
        density=LOW_SIM_POINTS_DENSITY;
    m_componentsList[numComp].setSimPointsDensity(density);
}
void Circuit::smoothRenderComponentSimPointsDensity(int numComp)
{
    m_componentsList[numComp].setSimPointsDensity(SMOOTH_RENDER_SIM_POINTS_DENSITY);
}


QVector<complexF> Circuit::getABCDmatrixComponent(int numComp, double _freqGHz, double _parameterRatio)
{
    return m_componentsList[numComp].getABCDmatrix(_freqGHz,_parameterRatio);
}



void Circuit::setDefaultFrequency(double _frequencyGHz)
{
    m_defaultFrequencyGHz=_frequencyGHz;
}



void Circuit::generatePixmapsComponents(const int _width, const int _heigth)
{
    double widthF = (double)_width;
    double heigthF = (double)_heigth;

    //qDebug() << "Drawing";

    for (int compType=0 ; compType<=8 ; compType++)
    {
        if (pixmapComponents[compType]->isNull()==false)
            delete pixmapComponents[compType];
        pixmapComponents[compType] = new QPixmap(_width,_heigth);
        pixmapComponents[compType]->fill(Qt::white);
        QPainter painterPixmap(pixmapComponents[compType]);

        if (compType==CAPA_S)
        {
            // ----- DRAW SERIE CAPACITOR ------
            // 4 lines for capa
            painterPixmap.setPen(QPen(QColor("black"), widthF/20.0, Qt::SolidLine, Qt::FlatCap));
            painterPixmap.drawLine(widthF*4.0/32.0,heigthF/2.0,widthF*13.0/32.0,heigthF/2.0);
            painterPixmap.drawLine(widthF*13.0/32.0,heigthF*8.0/32.0,widthF*13.0/32.0,heigthF*24.0/32.0);
            painterPixmap.drawLine(widthF*19.0/32.0,heigthF*8.0/32.0,widthF*19.0/32.0,heigthF*24.0/32.0);
            painterPixmap.drawLine(widthF*19.0/32.0,heigthF/2.0,widthF*28.0/32.0,heigthF/2.0);
        }
        else if (compType==SELF_S)
        {
            // ----- DRAW SERIE SELF ------
            // 2 lines and 3 arc for self
            painterPixmap.setPen(QPen(QColor("black"), widthF/20.0, Qt::SolidLine, Qt::RoundCap));
            painterPixmap.drawLine(widthF*3.0/32.0,heigthF/2.0,widthF*7.0/32.0,heigthF/2.0);
            painterPixmap.drawArc((int)(widthF*7.0/32.0) , (int)(heigthF*12.0/32.0) , (int)(widthF*6.0/32.0) , (int)(heigthF*8.0/32.0) , 2880 , -2880);
            painterPixmap.drawArc((int)(widthF*13.0/32.0) , (int)(heigthF*12.0/32.0) , (int)(widthF*6.0/32.0) , (int)(heigthF*8.0/32.0) , 2880 , -2880);
            painterPixmap.drawArc((int)(widthF*19.0/32.0) , (int)(heigthF*12.0/32.0) , (int)(widthF*6.0/32.0) , (int)(heigthF*8.0/32.0) , 2880 , -2880);
            painterPixmap.drawLine(widthF*25.0/32.0,heigthF/2.0,widthF*29.0/32.0,heigthF/2.0);
        }
        else if (compType==RESIST_S)
        {
            // ----- DRAW SERIE RESISTOR ------
            // 1 path for resistor
            painterPixmap.setPen(QPen(QColor("black"), widthF/20.0, Qt::SolidLine, Qt::RoundCap));
            QPainterPath path1;
            path1.moveTo(widthF*3.0/32.0,heigthF*16.0/32.0);
            path1.lineTo(widthF*8.0/32.0,heigthF*16.0/32.0);
            path1.lineTo(widthF*10.0/32.0,heigthF*10.0/32.0);
            path1.lineTo(widthF*13.0/32.0,heigthF*22.0/32.0);
            path1.lineTo(widthF*16.0/32.0,heigthF*10.0/32.0);
            path1.lineTo(widthF*19.0/32.0,heigthF*22.0/32.0);
            path1.lineTo(widthF*22.0/32.0,heigthF*10.0/32.0);
            path1.lineTo(widthF*24.0/32.0,heigthF*16.0/32.0);
            path1.lineTo(widthF*29.0/32.0,heigthF/2.0);
            painterPixmap.drawPath(path1);
        }

        else if (compType==LINE_S)
        {
            // ----- DRAW TRANSMISSION LINE ------
            painterPixmap.setPen(QPen(QColor("black"), widthF/30.0, Qt::SolidLine, Qt::FlatCap));
            painterPixmap.drawLine(widthF*3.0/32.0,heigthF*15.0/32.0,widthF*8.0/32.0,heigthF*15.0/32.0);
            painterPixmap.drawRect(widthF*8.0/32.0, heigthF*12.0/32.0,widthF*16.0/32.0, heigthF*6.0/32.0);
            painterPixmap.drawLine(widthF*24.0/32.0,heigthF*15.0/32.0,widthF*29.0/32.0,heigthF*15.0/32.0);

            //gnd lines
            painterPixmap.setPen(QPen(QColor("black"), widthF/40.0, Qt::SolidLine, Qt::FlatCap));
            painterPixmap.drawLine(widthF*3.0/32.0,heigthF*23.0/32.0,widthF*29.0/32.0,heigthF*23.0/32.0);

            painterPixmap.drawLine(widthF*9.0/32.0,heigthF*23.0/32.0,widthF*5.0/32.0,heigthF*26.0/32.0);
            painterPixmap.drawLine(widthF*15.0/32.0,heigthF*23.0/32.0,widthF*11.0/32.0,heigthF*26.0/32.0);
            painterPixmap.drawLine(widthF*21.0/32.0,heigthF*23.0/32.0,widthF*17.0/32.0,heigthF*26.0/32.0);
            painterPixmap.drawLine(widthF*27.0/32.0,heigthF*23.0/32.0,widthF*23.0/32.0,heigthF*26.0/32.0);
        }
        else if (compType==CAPA_P)
        {
            // ----- DRAW PARALLEL CAPACITOR ------
            painterPixmap.setPen(QPen(QColor("black"), widthF/20.0, Qt::SolidLine, Qt::FlatCap));
            painterPixmap.rotate(90.0);
            painterPixmap.translate(0 , -heigthF);
            painterPixmap.drawLine(widthF*4.0/32.0,heigthF/2.0,widthF*13.0/32.0,heigthF/2.0);
            painterPixmap.drawLine(widthF*13.0/32.0,heigthF*8.0/32.0,widthF*13.0/32.0,heigthF*24.0/32.0);
            painterPixmap.drawLine(widthF*19.0/32.0,heigthF*8.0/32.0,widthF*19.0/32.0,heigthF*24.0/32.0);
            painterPixmap.drawLine(widthF*19.0/32.0,heigthF/2.0,widthF*26.0/32.0,heigthF/2.0);
            painterPixmap.translate(0 , heigthF);
            painterPixmap.rotate(-90.0);

            //gnd lines
            painterPixmap.drawLine(widthF*5.0/32.0,heigthF*26.0/32.0,widthF*26.0/32.0,heigthF*26.0/32.0);
            painterPixmap.drawLine(widthF*11.0/32.0,heigthF*26.0/32.0,widthF*7.0/32.0,heigthF*28.0/32.0);
            painterPixmap.drawLine(widthF*17.0/32.0,heigthF*26.0/32.0,widthF*13.0/32.0,heigthF*28.0/32.0);
            painterPixmap.drawLine(widthF*23.0/32.0,heigthF*26.0/32.0,widthF*19.0/32.0,heigthF*28.0/32.0);
        }
        else if (compType==SELF_P)
        {
            // ----- DRAW PARALLEL SELF ------
            painterPixmap.setPen(QPen(QColor("black"), widthF/20.0, Qt::SolidLine, Qt::RoundCap));
            painterPixmap.rotate(90.0);
            painterPixmap.translate(0 , -heigthF);
            painterPixmap.drawLine(widthF*3.0/32.0,heigthF/2.0,widthF*7.0/32.0,heigthF/2.0);
            painterPixmap.drawArc((int)(widthF*7.0/32.0) , (int)(heigthF*12.0/32.0) , (int)(widthF*5.0/32.0) , (int)(heigthF*8.0/32.0) , 2880 , -2880);
            painterPixmap.drawArc((int)(widthF*12.0/32.0) , (int)(heigthF*12.0/32.0) , (int)(widthF*5.0/32.0) , (int)(heigthF*8.0/32.0) , 2880 , -2880);
            painterPixmap.drawArc((int)(widthF*18.0/32.0) , (int)(heigthF*12.0/32.0) , (int)(widthF*5.0/32.0) , (int)(heigthF*8.0/32.0) , 2880 , -2880);
            painterPixmap.drawLine(widthF*23.0/32.0,heigthF/2.0,widthF*26.0/32.0,heigthF/2.0);
            painterPixmap.translate(0 , heigthF);
            painterPixmap.rotate(-90.0);

            //gnd lines
            painterPixmap.drawLine(widthF*5.0/32.0,heigthF*26.0/32.0,widthF*26.0/32.0,heigthF*26.0/32.0);
            painterPixmap.drawLine(widthF*11.0/32.0,heigthF*26.0/32.0,widthF*7.0/32.0,heigthF*28.0/32.0);
            painterPixmap.drawLine(widthF*17.0/32.0,heigthF*26.0/32.0,widthF*13.0/32.0,heigthF*28.0/32.0);
            painterPixmap.drawLine(widthF*23.0/32.0,heigthF*26.0/32.0,widthF*19.0/32.0,heigthF*28.0/32.0);
        }
        else if (compType==RESIST_P)
        {
            // ----- DRAW PARALLEL RESISTOR ------
            painterPixmap.setPen(QPen(QColor("black"), widthF/20.0, Qt::SolidLine, Qt::RoundCap));
            painterPixmap.rotate(90.0);
            painterPixmap.translate(0 , -heigthF);
            QPainterPath path2;
            path2.moveTo(widthF*3.0/32.0,heigthF*16.0/32.0);
            path2.lineTo(widthF*8.0/32.0,heigthF*16.0/32.0);
            path2.lineTo(widthF*9.0/32.0,heigthF*12.0/32.0);
            path2.lineTo(widthF*12.0/32.0,heigthF*20.0/32.0);
            path2.lineTo(widthF*15.0/32.0,heigthF*12.0/32.0);
            path2.lineTo(widthF*18.0/32.0,heigthF*20.0/32.0);
            path2.lineTo(widthF*21.0/32.0,heigthF*12.0/32.0);
            path2.lineTo(widthF*23.0/32.0,heigthF*16.0/32.0);
            path2.lineTo(widthF*25.0/32.0,heigthF/2.0);
            painterPixmap.drawPath(path2);
            painterPixmap.translate(0 , heigthF);
            painterPixmap.rotate(-90.0);

            //gnd lines
            painterPixmap.drawLine(widthF*5.0/32.0,heigthF*26.0/32.0,widthF*26.0/32.0,heigthF*26.0/32.0);
            painterPixmap.drawLine(widthF*11.0/32.0,heigthF*26.0/32.0,widthF*7.0/32.0,heigthF*28.0/32.0);
            painterPixmap.drawLine(widthF*17.0/32.0,heigthF*26.0/32.0,widthF*13.0/32.0,heigthF*28.0/32.0);
            painterPixmap.drawLine(widthF*23.0/32.0,heigthF*26.0/32.0,widthF*19.0/32.0,heigthF*28.0/32.0);
        }
        else if (compType==LINE_OPEN)
        {
            // ----- DRAW OPEN STUB ------
            painterPixmap.setPen(QPen(QColor("black"), widthF/30.0, Qt::SolidLine, Qt::FlatCap));
            painterPixmap.drawLine(widthF*3.0/32.0,heigthF*15.0/32.0,widthF*8.0/32.0,heigthF*15.0/32.0);
            painterPixmap.drawRect(widthF*8.0/32.0, heigthF*12.0/32.0,widthF*16.0/32.0, heigthF*6.0/32.0);
            //painterPixmap.drawLine(widthF*24.0/32.0,heigthF*15.0/32.0,widthF*29.0/32.0,heigthF*15.0/32.0);
            //painterPixmap.drawLine(widthF*29.0/32.0,heigthF*13.0/32.0,widthF*29.0/32.0,heigthF*18.0/32.0);

            //gnd lines
            painterPixmap.setPen(QPen(QColor("black"), widthF/40.0, Qt::SolidLine, Qt::FlatCap));
            painterPixmap.drawLine(widthF*3.0/32.0,heigthF*23.0/32.0,widthF*29.0/32.0,heigthF*23.0/32.0);

            painterPixmap.drawLine(widthF*9.0/32.0,heigthF*23.0/32.0,widthF*5.0/32.0,heigthF*26.0/32.0);
            painterPixmap.drawLine(widthF*15.0/32.0,heigthF*23.0/32.0,widthF*11.0/32.0,heigthF*26.0/32.0);
            painterPixmap.drawLine(widthF*21.0/32.0,heigthF*23.0/32.0,widthF*17.0/32.0,heigthF*26.0/32.0);
            painterPixmap.drawLine(widthF*27.0/32.0,heigthF*23.0/32.0,widthF*23.0/32.0,heigthF*26.0/32.0);
        }
        else if (compType==LINE_SHORT)
        {
            // ----- DRAW SHORT STUB ------
            painterPixmap.setPen(QPen(QColor("black"), widthF/30.0, Qt::SolidLine, Qt::FlatCap));
            painterPixmap.drawLine(widthF*3.0/32.0,heigthF*15.0/32.0,widthF*8.0/32.0,heigthF*15.0/32.0);
            painterPixmap.drawRect(widthF*8.0/32.0, heigthF*12.0/32.0,widthF*14.0/32.0, heigthF*6.0/32.0);
            painterPixmap.drawLine(widthF*22.0/32.0,heigthF*15.0/32.0,widthF*26.0/32.0,heigthF*15.0/32.0);
            painterPixmap.drawLine(widthF*26.0/32.0,heigthF*15.0/32.0,widthF*26.0/32.0,heigthF*23.0/32.0);

            //gnd lines
            painterPixmap.setPen(QPen(QColor("black"), widthF/40.0, Qt::SolidLine, Qt::FlatCap));
            painterPixmap.drawLine(widthF*3.0/32.0,heigthF*23.0/32.0,widthF*29.0/32.0,heigthF*23.0/32.0);

            painterPixmap.drawLine(widthF*9.0/32.0,heigthF*23.0/32.0,widthF*5.0/32.0,heigthF*26.0/32.0);
            painterPixmap.drawLine(widthF*15.0/32.0,heigthF*23.0/32.0,widthF*11.0/32.0,heigthF*26.0/32.0);
            painterPixmap.drawLine(widthF*21.0/32.0,heigthF*23.0/32.0,widthF*17.0/32.0,heigthF*26.0/32.0);
            painterPixmap.drawLine(widthF*27.0/32.0,heigthF*23.0/32.0,widthF*23.0/32.0,heigthF*26.0/32.0);
        }

    // rectangle for border
    painterPixmap.setPen(QPen(QColor("black"), widthF/40.0, Qt::SolidLine, Qt::FlatCap));
    painterPixmap.drawRect(widthF*0.5/32.0,heigthF*0.5/32.0,widthF*31.0/32.0,heigthF*31.0/32.0);

    painterPixmap.end();
    }

    // Pixmap : the '+' button to add comp
    if (pixmapSuppComp->isNull()==false)
        delete pixmapSuppComp;
    pixmapSuppComp = new QPixmap(_width,_heigth);
    pixmapSuppComp->fill(Qt::white);
    QPainter painterPixmap1(pixmapSuppComp);
    painterPixmap1.setPen(QPen(QColor("black"), widthF/20.0, Qt::SolidLine, Qt::FlatCap));
    painterPixmap1.drawLine(widthF*16.0/32.0,heigthF*5.0/32.0,widthF*16.0/32.0,heigthF*27.0/32.0);
    painterPixmap1.drawLine(widthF*5.0/32.0,heigthF*16.0/32.0,widthF*27.0/32.0,heigthF*16.0/32.0);
    painterPixmap1.setPen(QPen(QColor("black"), widthF/40.0, Qt::SolidLine, Qt::FlatCap));
    painterPixmap1.drawRect(widthF*0.5/32.0,heigthF*0.5/32.0,widthF*31.0/32.0,heigthF*31.0/32.0);
    painterPixmap1.end();

    // Pixmap : the dot line and the point representing start point and end point in the circuit display (mainWindow)
    if (pixmapDotAndPoint->isNull()==false)
        delete pixmapDotAndPoint;
    pixmapDotAndPoint = new QPixmap(_width,_heigth);
    pixmapDotAndPoint->fill(Qt::white);
    QPainter painterPixmap2(pixmapDotAndPoint);
    painterPixmap2.setPen(QPen(QColor("black"), widthF/5.0, Qt::SolidLine, Qt::RoundCap));
    painterPixmap2.drawPoint(widthF*16.0/32.0,heigthF*16.0/32.0);
    painterPixmap2.setPen(QPen(QColor("black"), widthF/20.0, Qt::DashLine, Qt::FlatCap));
    painterPixmap2.drawLine(widthF*16.0/32.0,heigthF*16.0/32.0,widthF*16.0/32.0,heigthF*30.0/32.0);
    //painterPixmap.drawLine(widthF*5.0/32.0,heigthF*16.0/32.0,widthF*27.0/32.0,heigthF*16.0/32.0);

    painterPixmap2.end();
}




