/****************************************************************************
 * QSmithTool :                                                             *
 *   Copyright (C) 2015 by Sébastien Lissarre                               *
 *   sebastien.lissarre@gmail.com                                           *
 *                                                                          *
 * Smith Chart drawing base :                                               *
 *   Copyright (C) 2008 by Lisandro Damián Nicanor Pérez Meyer              *
 *   perezmeyer@gmail.com                                                   *
 *                                                                          *
 *                                                                          *
 *  This file is part of QSmithTool.                                        *
 *                                                                          *
 *  QSmithTool is free software: you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation, either version 3 of the License, or       *
 *  (at your option) any later version.                                     *
 *                                                                          *
 *  QSmithTool is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with QSmithTool.  If not, see <http://www.gnu.org/licenses/>.     *
 ***************************************************************************/


#ifndef CIRCUIT_H
#define CIRCUIT_H

#include <QPixmap>
#include <QPainter>

#include "component.h"
#include "rfFctsLibrary.h"


/**
   This class instanciate a list of "Component" items and allow the user to interact with them
*/
class Circuit
{
public:
    Circuit(double _frequencyGHz);

    /**
       @brief Instanciate a new circuit
       @param _frequencyGhz is the frequency in GHz used to add an appropriate default value to new components
       @return None
    */
    void addComponent(Component comp, bool componentPreview=false);
    void addComponent(ComponentType _compType, CompParametersList _parameters, bool _componentPreview=false );

    /**
       @brief Add a component with default value
       @param type is the component type (CAPA_S, CAPA_P, LINE_S, SHORT_STUB, etc.)
       @param componentPreview is true if the component is in preview state (grayed)
       @return None
    */
    void addDefaultComponent(ComponentType type, bool componentPreview=false);

    /**
       @brief Insert a component before another in the circuit
       @param numCompBefore is the component index position after wich we want to insert the new component
       @return None
    */
    void insertCAPA_S(int numCompBefore);
    void insertSELF_S(int numCompBefore);
    void insertRESIST_S(int numCompBefore);
    void insertLINE_S(int numCompBefore);
    void insertCAPA_P(int numCompBefore);
    void insertSELF_P(int numCompBefore);
    void insertRESIST_P(int numCompBefore);
    void insertLINE_OPEN(int numCompBefore);
    void insertLINE_SHORT(int numCompBefore);

    /**
       @brief Remove a component in circuit
       @param numComp is the component index position
       @return None
    */
    void removeComponent(int numComp);

    /**
       @brief Remove the last component in circuit
       @param None
       @return None
    */
    void removeLastComponent();

    /**
       @brief Replace a component by another
       @param numComp is the index position of the component to replace
       @param comp is the new component
       @return None
    */
    void replaceComponent(int numComp, Component comp);

    /**
       @brief Remove all components from circuit
       @param None
       @return None
    */
    void clearAll();

    /**
       @brief Add or modify (if existing) the preview component (grayed on chart)
       @param type is the component type (CAPA_S, CAPA_P, LINE_S, SHORT_STUB, etc.)
       @return None
    */
    void setPreviewComponent(ComponentType type);

    /**
       @brief Return the type of the preview component in circuit (CAPA_S, CAPA_P, LINE_S,...)
       @param None
       @return component type
    */
    ComponentType getPreviewComponentType();

    /**
       @brief Delete the component in preview state if already existing
       @param None
       @return None
    */
    void removePreviewComponent();

    /**
       @brief Add a component with default value
       @param None
       @return None
    */
    void addNewCAPA_S (bool componentPreview);
    void addNewSELF_S (bool componentPreview);
    void addNewRESIST_S (bool componentPreview);
    void addNewCAPA_P (bool componentPreview);
    void addNewSELF_P (bool componentPreview);
    void addNewRESIST_P (bool componentPreview);
    void addNewLINE_S (bool componentPreview);
    void addNewLINE_OPEN (bool componentPreview);
    void addNewLINE_SHORT (bool componentPreview);

    /**
       @brief Get a const copy of a component
       @param numComp is the component index position
       @return The copy of component
    */
    Component getComponentCopy(int numComp) const;

    /**
        @brief Get a const list of the components in circuit
        @param None
        @return The component list (QVector<Component>)
    */
    ComponentsList getComponentsList() const;

    /**
        @brief Count the number of components in circuit
        @param None
        @return int
    */
    int getSize();

    /**
        @brief Count the number of components in circuit without the last one if it is in preview state
        @param None
        @return int
    */
    int getSizeWithoutPreviewed();

    /**
        @brief Count the total number of data points of each component in circuit
        @param None
        @return int
    */
    int getTotalNumberOfPoints();
    /**
        @brief Return a data points list of all points calculated for this component
        @param None
        @return data points list (QVector<QVector<complexF>)
    */
    QVector< QVector<complexF> > & getComponentDataPoints(int numComp);

    /**
        @brief Add a list of data points for the component numComp
        @param numComp is the component index position
        @param ZVector is the list of points to add
        @return None
    */
    void addComponentDataPoints(int numComp, int numStartPoint, QVector<complexF> &pointVector);

    /**
        @brief Delete all data points for the component with index position numComp
        @param numComp is the component index position in circuit
        @return None
    */
    void clearComponentDataPoints(int numComp);
    void clearComponentDataPoints(int numComp, int numStartPoint);

    /**
        @brief Delete all data points for all the components
        @param None
        @return None
    */
    void clearAllDataPoints();

    /**
        @brief Return the last point which not belong to a preview component
        @param defaultReturn is the default value returned if not found (like no point calculated
        @return The point impedance
    */
    complexF getLastPointNotInPreview(int numStartPoint = 0, complexF defaultReturn = complexF(1.0,0.0));

    /**
        @brief Get the component name ("Serial inductor", "Parallel capacitor",...)
        @param numComp is the component index position in circuit
        @return The string in QString format
    */
    QString getComponentName(int numComp) const;

    /**
        @brief Get the component type (SELF_S, CAPA_P, LINE_SHORT, ...)
        @param numComp is the component index position in circuit
        @return The type
    */
    ComponentType getComponentType(int numComp) const;

    /**
        @brief Get the component unit string ("nH", "pF", "mm", ...)
        @param numComp is the component index position in circuit
        @return The string in QString format
    */
    QString getComponentStrUnit(int numComp) const;

    /**
        @brief Set/get functions for all component parameters
        @param numComp is the component index position in circuit
        @param Z0 is the reference impedance to store parameters in normalized format (50ohm=>1ohm)
    */
    double getComponentParameter(int numComp,int numParam) const;
    CompParametersList getComponentParameters(int numComp) const;
    CompParametersList getComponentParametersDenorm(int numComp, double Z0);
    double getComponentParameterDenorm(int numComp, int numParam, double Z0);
    void setComponentParameterDenorm(int numComp,int numParam, double param, double Z0);
    bool getComponentPreviewState(int numComp) const;
    void setComponentPreviewState(int numComp, bool state);
    bool getComponentMouseOverState(int numComp) const;
    void setComponentMouseOverState(int numComp, bool state);
    void resetAllComponentsMouseOverState();

    /**
        @brief Apply a ratio on a parameter (for example, to increase capacitor value by 20%)
        @param numComp is the component index position in circuit
        @param ratio is the ratio between 0 and 1
        @return None
    */
    void ratioOnCompFirstParameter(int numComp, double ratio);

    /**
        @brief Return the pictures generated for a component or the '+' picture
        @param type is the component type (see corresponding enum)
        @param _width/_heigth are the size of the pixmap, if not corresponding to already generated, regenerate it with good size
        @return The pixmap
    */
    QPixmap getPixmapComponent(ComponentType type, int _width, int _heigth);
    QPixmap getPixmapSuppComp(int _width, int _heigth);
    QPixmap getPixmapDotAndPoint(int _width, int _heigth);

    /**
     * @brief getComponentSimPointsDensity
     * @return nb of points to simulate
     */
    int getComponentSimPointsDensity(int _numComp) const;
    void setComponentSimPointsDensity(int _numComp, int _density);
    /**
     * @brief increaseComponentSimPointsDensity
     * @param _numComp
     */
    void increaseComponentSimPointsDensity(int _numComp);
    /**
     * @brief decreaseComponentSimPointsDensity
     * @param _numComp
     */
    void decreaseComponentSimPointsDensity(int _numComp);
    /**
     * @brief Set the sim point density to 1000 (take few secs to calculate)
     * @param _numComp
     */
    void smoothRenderComponentSimPointsDensity(int _numComp);

    /**
     * @brief getABCDmatrixComponent return the ABCD matrix parameters to calculate S11 or S21
     * @param numComp is the component index position in circuit
     * @param _freqGHz frequency in GHz to gen. ABCD matrix
     * @return
     */
    QVector<complexF> getABCDmatrixComponent(int numComp, double _freqGHz, double _parameterRatio);

    /**
        @brief Modify the frequency used to calculate default value for new component
        @param _frequencyGHz is the frequency in GHz
        @return None
    */
    void setDefaultFrequency(double _frequencyGHz);


private:
    /// Generate new pixmap for all component pictures
    void generatePixmapsComponents(const int _width, const int _heigth);

    /// Contain the list of components in the circuit
    ComponentsList m_componentsList;
    /// Pictures generated for each components
    QPixmap *pixmapComponents[NB_COMP_TYPE];
    /// Picture of the '+' pixmap
    QPixmap *pixmapSuppComp;
    /// Picture for the start point & end point in the circuit display widget (the QListWidget in mainwindow)
    QPixmap *pixmapDotAndPoint;

    /// Only used to add new component with suited default value
    double m_defaultFrequencyGHz;
};

#endif // CIRCUIT_H
