
#include "circuitSimulation.h"


void circuitSimulator::startSimulation(const QVector<complexFAndFreq> &_startPoints, Circuit &_circuit, const SimQuality _simQuality)
{
    complexFAndFreq startPoint;

    qDebug() << ">-------------SIMULATION----------------";
    QElapsedTimer timerElaps;
    qint64 ns_elaps=0;
    double ms_elaps=0.0;

    if (( _circuit.getSize() != 0 ) && (_startPoints.size() != 0))
    {
        // Measure simulation time
        timerElaps.start();

        for (int numStartPoint=0 ; numStartPoint<_startPoints.size() ; numStartPoint++) // for each start point
        {
            #ifdef _SIM_QDEBUG
                qDebug() << "--- Sim : start point n° " << QString::number(numStartPoint) << "---";
                qDebug() << "Frequency : " << _startPoints[numStartPoint].freq << "GHz";
            #endif

            startPoint = _startPoints[numStartPoint];
            circuitSimulation(startPoint, numStartPoint, _circuit, _simQuality);
        }
        ns_elaps = timerElaps.nsecsElapsed();
    }
    else
        qDebug() << "Not enough component or start points to simulate";


    qDebug() << ">----------END OF SIMULATION------------";


    ms_elaps = ns_elaps/1000000.0;
    qDebug() << "Total sim calculation time :" << ms_elaps << "ms";
}


void circuitSimulator::circuitSimulation(const complexFAndFreq _startPoint, int _numStartPoint, Circuit &_circuit, const SimQuality _simQuality)
{
    /// Scale of component parameter value generated with linearScale() &  invNonLinearScale()
    CompParametersList parameterRatio;
    /// Vector of points calculated for this component
    QVector<complexF> resultPoints;

    #ifdef _SIM_QDEBUG
        QElapsedTimer timerElapsAll, timerElapsZin;
        qint64 ns_elaps;
        timerElapsAll.start(); // timer to measure sim time for the current component
    #endif

    QVector<complexF> matrixRes; matrixRes.append(complexF(1.0,0.0)); matrixRes.append(complexF(0.0,0.0)); matrixRes.append(complexF(0.0,0.0)); matrixRes.append(complexF(1.0,0.0));

    for (int numComp=0 ; numComp<_circuit.getSize() ; numComp++) // Calculate point for each component
    {
        if (numComp>=1) {
            QVector<complexF> matrixTmp1 = _circuit.getABCDmatrixComponent(numComp-1, _startPoint.freq, 1.0);
            matrixRes = matrixProduct(matrixRes, matrixTmp1);
        }

        // Get the number of points to simulate
        int nbSimPoints=0;
        if (_simQuality==QUICK_SIM)
            nbSimPoints = LOW_SIM_POINTS_DENSITY; // quick simulation
        else if (_simQuality==HQ_SIM)
            nbSimPoints = SMOOTH_RENDER_SIM_POINTS_DENSITY; // high quality simulation
        else { // standard quality simulation
            nbSimPoints =  _circuit.getComponentSimPointsDensity(numComp);
            if (nbSimPoints==SMOOTH_RENDER_SIM_POINTS_DENSITY) // if a render is done, reset the sim point density for next simulation
                _circuit.setComponentSimPointsDensity(numComp, HIGH_SIM_POINTS_DENSITY);
        }
        #ifdef _SIM_QDEBUG
            qDebug() << "Simulation density : " <<  nbSimPoints;
        #endif

        // Load a scale with each value to simulate for the component (for CAP_S of 20pF, preCalcParamList[i]=20pF,18pF,16pF,14pF,...,0pF)
        parameterRatio.resize(nbSimPoints);
        ComponentType compType = _circuit.getComponentType(numComp);
        double compParameter0 = _circuit.getComponentParameter(numComp, 0); // pF, nH, ohm or line length
        if ((compType==CAPA_S)||(compType==SELF_P)||(compType==RESIST_P))
            for (int i=0 ; i<nbSimPoints ; i++)
               parameterRatio[i] = invNonLinearScale(1.0, nbSimPoints, i);
        else if ((compType==LINE_SHORT)) // need linear scale from lambda/4 to stub length
        {
            double compParameter2 = _circuit.getComponentParameter(numComp, 2); // permitivity
            double Llambda4 = 300.0/(4.0*_startPoint.freq)/ sqrt(compParameter2); // Lambda/4 en mm = 3.0e8/(4*freqHz) * 1000
            for (int i=0 ; i<nbSimPoints ; i++)
               parameterRatio[i] = linearScale(Llambda4/compParameter0, 1.0, nbSimPoints, i);
        }
        else
            for (int i=0 ; i<nbSimPoints ; i++)
                parameterRatio[i] = linearScale(0.0, 1.0, nbSimPoints, i);

        // Loop for every component value
        #ifdef _SIM_QDEBUG
            timerElapsZin.start(); // timer for the real simulation kernel : Zin calculation
        #endif
        resultPoints.resize(nbSimPoints);

        for (int i=0 ; i<nbSimPoints ; i++)
        {
            QVector<complexF> matrixTmp2 = _circuit.getABCDmatrixComponent(numComp, _startPoint.freq, parameterRatio[i]);
            QVector<complexF> matrixRes2 = matrixProduct(matrixRes, matrixTmp2);
            //complexF myS11 = myABCDtoS11(matrixRes);
            //double myS11dB = myS11toS11dB(myS11);
            if ((numComp==_circuit.getSize()-1)&&(i==nbSimPoints-1)) {
                double myS21dB = myABCDtoS21dB(matrixRes2, _startPoint.point);
                qDebug() << "Total S21 = " << myS21dB << "dB";
            }
            resultPoints[i] = (matrixRes2[3]*_startPoint.point+matrixRes2[1])/(matrixRes2[2]*_startPoint.point+matrixRes2[0]);
        }
        #ifdef _SIM_QDEBUG
            ns_elaps = timerElapsZin.nsecsElapsed();
            qDebug() << "Zin calculation time :" << ns_elaps/1000.0 << "us";
        #endif

        // Check if simulation point density is enough, else resimulate with density x2 the next time
        if (_simQuality==STANDARD_SIM) {
            if (nbSimPoints>=2) {
                QPointF distXYfirst_p, distXYlast_p;
                distXYfirst_p = SmithChart::ZtoXYPainterCalc(resultPoints[0])-SmithChart::ZtoXYPainterCalc(_startPoint.point);
                distXYlast_p = SmithChart::ZtoXYPainterCalc(resultPoints[nbSimPoints-1])-SmithChart::ZtoXYPainterCalc(resultPoints[nbSimPoints-2]);
                #ifdef _SIM_QDEBUG
                    qDebug() << "Distance first in painter : "  << distXYfirst_p << distXYfirst_p.manhattanLength();
                    qDebug() << "Distance last in painter : "  << distXYlast_p << distXYlast_p.manhattanLength();
                #endif

                if ((distXYfirst_p.manhattanLength()>50)||(distXYlast_p.manhattanLength()>50))
                {
                    #ifdef _SIM_QDEBUG
                        qDebug() << "Increasing simulation density.";
                    #endif
                    _circuit.increaseComponentSimPointsDensity(numComp);
                }
                else if ((distXYfirst_p.manhattanLength()<25)&&(distXYlast_p.manhattanLength()<25))
                {
                    #ifdef _SIM_QDEBUG
                        qDebug() << "Decreasing simulation density.";
                    #endif
                    _circuit.decreaseComponentSimPointsDensity(numComp);
                }
            }
        }

        _circuit.clearComponentDataPoints(numComp, _numStartPoint);
        _circuit.addComponentDataPoints(numComp, _numStartPoint, resultPoints);
    }

    #ifdef _SIM_QDEBUG
        ns_elaps = timerElapsAll.nsecsElapsed();
        qDebug() << "Comp sim time outside :" << ns_elaps/1000000.0 << "ms";
    #endif
}


// linearScale(0.0, 10.0, 10, x) => return 1.0, 2.0, ... , 10.0
double circuitSimulator::linearScale(const double start, const double stop, int nbVal, int numVal)
{
    double res=0.0;
    res = (stop-start)/nbVal * (numVal+1) + start;
    //qDebug() << "Lin val : " << res;
    return res;
}

// nonLinearScale1(10.0, 10, x) => return 100.0, 50.0, 33.3, 25.0, 20.0,..., 11.1, 10.0
double circuitSimulator::invNonLinearScale(const double lastVal, int nbVal, int numVal)
{
    // res = 32*(firstVal)^-1.0
    double res=0.0;
    double firstVal = lastVal*nbVal;
    res = firstVal*pow(numVal+1, -1.0);
    //qDebug() << "Log val : " << res;
    return res;
}


