#include "component.h"


Component::Component()
{
    m_type = NOT_DEF;
}

Component::Component(ComponentType type, double parameter)
{
    m_type = type;
    m_name = compName[type];
    m_strUnit = compStrUnit[type];
    m_parameters = CompParametersList (1, parameter);
    m_simPointsDensity = HIGH_SIM_POINTS_DENSITY;

    m_previewState = false;
    m_mouseOverState = false;
    m_dataPointsZ.clear();

//    for (int i=0 ; i<4 ; i++)
//        ABCDmatrix[i]=complexF(0.0,0.0);
}


Component::Component(ComponentType type, CompParametersList parameters)
{
    m_type = type;
    m_name = compName[type];
    m_strUnit = compStrUnit[type];
    m_parameters = parameters;
    m_simPointsDensity = HIGH_SIM_POINTS_DENSITY;

    m_previewState = false;
    m_mouseOverState = false;
    m_dataPointsZ.clear();

//    m_ABCDmatrix.resize(4);
//    for (int i=0 ; i<4 ; i++)
//        m_ABCDmatrix[i]=complexF(0.0,0.0);
}


void Component::clearDataPoints()
{
    m_dataPointsZ.clear();
}
void Component::clearDataPoints(int numStartPoint)
{
    if (m_dataPointsZ.size() >= numStartPoint+1)
        m_dataPointsZ[numStartPoint].clear();
}
void Component::addDataPoints(int numStartPoint, QVector <complexF> &pointVector)
{
    // Check if the point array is big enough
    if (m_dataPointsZ.size()<numStartPoint+1)
        m_dataPointsZ.resize(numStartPoint+1);

    // Add the normalized point in vector
    for (int i=0 ; i<pointVector.size() ; i++)
    {
        m_dataPointsZ[numStartPoint].append(pointVector[i]);
        //qDebug() << "Add point (norm) :" << complexNumberToQString( pointVector[i] );
    }
}
 QVector< QVector<complexF> > & Component::getDataPoints()
{

    if (m_dataPointsZ.size()==0)
        qDebug() << "Error : no data points loaded";
    else {
        if (m_dataPointsZ[0].size()==0)
         qDebug() << "Error : no data points loaded";
    }

    return m_dataPointsZ;
}


ComponentType Component::getType() const
{
    return m_type;
}


QString Component::getName() const
{
    return m_name;
}


QString Component::getStrUnit() const
{
    return m_strUnit;
}


void Component::setParameter(int numParam, double param)
{
    if (numParam<m_parameters.size())
        m_parameters.replace(numParam, param);
    else
        qDebug() << "setParameter() : numParam out of range";

}
void Component::setParameters(CompParametersList params)
{
    m_parameters = params;
}
void Component::ratioOnFirstParameter(double ratio)
{
    qDebug() << "Component" << getName() << " : Apply ratio on first paramter()";
    m_parameters[0] *= ratio;
}


CompParametersList Component::getParameters() const
{
    return m_parameters;
}
double Component::getParameter(int numParam) const
{
    return m_parameters[numParam];
}

bool Component::getPreviewState() const
{
    return m_previewState;
}
void Component::setPreviewState(bool state)
{
    m_previewState=state;
}
bool Component::getMouseOverState() const
{
    return m_mouseOverState;
}
void Component::setMouseOverState(bool state)
{
    m_mouseOverState=state;
}
int Component::getSimPointsDensity() const
{
    return m_simPointsDensity;
}
void Component::setSimPointsDensity(int _density)
{
    m_simPointsDensity=_density;
}

QVector<complexF> Component::getABCDmatrix(double _freqGHz, double _parameterRatio) const
{
    double electricalLength, alpha, beta;
    complexF gammaL, resCosh, resSinh, lineZ0;

    QVector<complexF> ABCDmatrix(4);
    //CAPA_S, SELF_S, RESIST_S, LINE_S, CAPA_P, SELF_P, RESIST_P, LINE_OPEN, LINE_SHORT
    switch (m_type)
    {
    case CAPA_S :
        ABCDmatrix[0]=complexF(1.0,0.0); ABCDmatrix[1]=complexF(0.0,-1.0/(2.0*_PI*_freqGHz*m_parameters[0]*1.0e-3*_parameterRatio)); ABCDmatrix[2]=complexF(0.0,0.0); ABCDmatrix[3]=complexF(1.0,0.0);
        break;
    case SELF_S :
        ABCDmatrix[0]=complexF(1.0,0.0); ABCDmatrix[1]=complexF(0.0,2.0*_PI*_freqGHz*m_parameters[0]*_parameterRatio); ABCDmatrix[2]=complexF(0.0,0.0); ABCDmatrix[3]=complexF(1.0,0.0);
        break;
    case RESIST_S :
        ABCDmatrix[0]=complexF(1.0,0.0); ABCDmatrix[1]=complexF(m_parameters[0]*_parameterRatio,0.0); ABCDmatrix[2]=complexF(0.0,0.0); ABCDmatrix[3]=complexF(1.0,0.0);
        break;
    case LINE_S :
        alpha = m_parameters[3]/8.68588; // loss factor dB/m => Neper/m
        beta = (2.0*_PI/3.0)*_freqGHz*10.0; // 2*PI/3e8 * 2e9 = 2*PI/3 * Ghz * 1e3
        lineZ0 = complexF(m_parameters[1], 0.0);
        electricalLength = m_parameters[0]*_parameterRatio * sqrt(m_parameters[2]); // electrical length = mechanical length * sqrt(Er)
        gammaL = complexF(alpha*electricalLength*1.0e-3, beta*electricalLength*1.0e-3); // 1e-3 because 'param' in mm
        resCosh = cosh(gammaL);
        resSinh = sinh(gammaL);
        ABCDmatrix[0]=resCosh; ABCDmatrix[1]=m_parameters[1]*resSinh; ABCDmatrix[2]=(1.0/m_parameters[1])*resSinh; ABCDmatrix[3]=resCosh;  // ABCD matrix of a serie transmission line
        break;
    case CAPA_P :
        ABCDmatrix[0]=complexF(1.0,0.0); ABCDmatrix[1]=complexF(0.0,0.0); ABCDmatrix[2]=complexF(0.0,2.0*_PI*_freqGHz*m_parameters[0]*1.0e-3*_parameterRatio); ABCDmatrix[3]=complexF(1.0,0.0);
       break;
    case SELF_P :
        ABCDmatrix[0]=complexF(1.0,0.0); ABCDmatrix[1]=complexF(0.0,0.0); ABCDmatrix[2]=complexF(0.0,-1.0/(2.0*_PI*_freqGHz*m_parameters[0]*_parameterRatio)); ABCDmatrix[3]=complexF(1.0,0.0);
        break;
    case RESIST_P :
        ABCDmatrix[0]=complexF(1.0,0.0); ABCDmatrix[1]=complexF(0.0,0.0); ABCDmatrix[2]=complexF(1.0/(m_parameters[0]*_parameterRatio), 0.0); ABCDmatrix[3]=complexF(1.0,0.0);
       break;
    case LINE_OPEN :
        alpha = m_parameters[3]/8.68588; // loss factor dB/m => Neper/m
        beta = (2.0*_PI/3.0)*_freqGHz*10.0; // 2*PI/3e8 * 2e9 = 2*PI/3 * Ghz * 1e3
        lineZ0 = complexF(m_parameters[1], 0.0);
        electricalLength = m_parameters[0]*_parameterRatio * sqrt(m_parameters[2]); // electrical length = mechanical length * sqrt(Er)
        gammaL = complexF(alpha*electricalLength*1.0e-3, beta*electricalLength*1.0e-3); // 1e-3 because 'param' in mm
        resCosh = cosh(gammaL);
        resSinh = sinh(gammaL);
        ABCDmatrix[0]=complexF(1.0, 0.0); ABCDmatrix[1]=complexF(0.0, 0.0); ABCDmatrix[2]=(1.0/lineZ0)*(resSinh/resCosh); ABCDmatrix[3]=complexF(1.0, 0.0);  // ABCD matrix of an open stub
        break;
    case LINE_SHORT :
        alpha = m_parameters[3]/8.68588; // loss factor dB/m => Neper/m
        beta = (2.0*_PI/3.0)*_freqGHz*10.0; // 2*PI/3e8 * 2e9 = 2*PI/3 * Ghz * 1e3
        lineZ0 = complexF(m_parameters[1], 0.0);
        electricalLength = m_parameters[0]*_parameterRatio * sqrt(m_parameters[2]); // electrical length = mechanical length * sqrt(Er)
        gammaL = complexF(alpha*electricalLength*1.0e-3, beta*electricalLength*1.0e-3); // 1e-3 because 'param' in mm
        resCosh = cosh(gammaL);
        resSinh = sinh(gammaL);
        ABCDmatrix[0]=complexF(1.0, 0.0); ABCDmatrix[1]=complexF(0.0, 0.0); ABCDmatrix[2]=(1.0/lineZ0)*(resCosh/resSinh); ABCDmatrix[3]=complexF(1.0, 0.0);  // ABCD matrix of a shorted line
        break;
    case NOT_DEF :
        qDebug() << "Error, getABCD matrix call on NOT_DEF component";
        break;
    }
    return ABCDmatrix;
}

