/****************************************************************************
 * QSmithTool :                                                             *
 *   Copyright (C) 2015 by Sébastien Lissarre                               *
 *   sebastien.lissarre@gmail.com                                           *
 *                                                                          *
 * Smith Chart drawing base :                                               *
 *   Copyright (C) 2008 by Lisandro Damián Nicanor Pérez Meyer              *
 *   perezmeyer@gmail.com                                                   *
 *                                                                          *
 *                                                                          *
 *  This file is part of QSmithTool.                                        *
 *                                                                          *
 *  QSmithTool is free software: you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation, either version 3 of the License, or       *
 *  (at your option) any later version.                                     *
 *                                                                          *
 *  QSmithTool is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with QSmithTool.  If not, see <http://www.gnu.org/licenses/>.     *
 ***************************************************************************/


#ifndef COMPONENT_H
#define COMPONENT_H

#include <QVector>
#include <QString>
#include <QDebug>
#include <QPointF>
#include <QColor>

#include "rfFctsLibrary.h"


/**
The class component can be manipulated only by the Circuit() class. It instanciate a component with
properties like the type of component (ComponentType), a list of parameters defining value of the component.
For a capacitor there is only one parameter (capacitance) and for a transmission line the list of parameters is
three value (Length, impedance, substrate permitivity). The data simulated are stored for each component in dataPointsZ.
*/
class Component
{
    friend class Circuit;

public:
    /// Not a public class, only a default constructor
    Component();

protected:
    Component(ComponentType type, double parameter);
    Component(ComponentType type, CompParametersList parameters);

    /**
     * @brief Use to get all the (normalized) impedances calculated for this component
     * @return Reference to the m_dataPointsZ array
     */
    QVector< QVector< complexF > > & getDataPoints();

    /**
     * @brief Clear data calculated for this component, for a particular start point or not
     */
    void clearDataPoints();
    void clearDataPoints(int numStartPoint);

    /**
     * @brief Add the array ZVector to the calculated points for this component
     * @param numStartPoint -> number of start point which for the data are calculated
     */
    void addDataPoints(int numStartPoint, QVector<complexF> &ZVector);

    /**
     * @brief Get the parameter(s) of the component (like capacitance for cap, length/imp for a line)
     * @param numParam
     * @return the parameter(s) for this component
     */
    double getParameter(int numParam) const;
    CompParametersList getParameters() const;

    /**
     * @brief Set the parameter(s) of the component (see also getParameter() )
     */
    void setParameter(int numParam, double param);
    void setParameters(CompParametersList params);
    /**
     * @brief Apply a ratio (like x0.3 or x1.6) on the component value
     */
    void ratioOnFirstParameter(double ratio);

    /**
     * @brief Return the type (like CAP, IND, RES), return the name (like "Serial capacitor") and unit (like "pF")
     */
    ComponentType getType() const;
    QString getName() const;
    QString getStrUnit() const;

    /**
     * @brief If the component is in preview state (in gray when user is adding a component with the mouse), return true
     */
    bool getPreviewState() const;
    void setPreviewState(bool state);
    /**
     * @brief If mouse is over the component, return true
     */
    bool getMouseOverState() const;
    void setMouseOverState(bool state);

    /**
     * @brief Get/set the simulation points density => number of point to simulate for this component (more pts == slower but smoother)
     */
    int getSimPointsDensity() const;
    void setSimPointsDensity(int _density);

    /**
     * @brief Return the ABCD matrix parameters for this component, not used in v1.3
     */
    QVector<complexF> getABCDmatrix(double _freqGHz, double _parameterRatio) const;

private:
    /// Contains all the data points simulated to draw on the chart
    /// dataPointsZ[StartPoint wo/ freq][Comp value]
    /// All the points are stocked in normalized value (50ohm basic point in 50ohm chart => Z=1.0+j0.0)
    QVector< QVector< complexF > > m_dataPointsZ;

    /// Component type (CAPA_S, CAPA_P,...)
    ComponentType m_type;
    /// Name of the component (ex : "Serial inductor")
    QString m_name;
    /// The unit in QString like "mm", "pF",...
    QString m_strUnit;
    /// List of electrical parameters : length, value, ...
    CompParametersList m_parameters;
    /// If the component is used as a preview (not appear in list and draw in gray)
    bool m_previewState;
    /// True if the user mouse is over the component
    bool m_mouseOverState;

    /// Contains the number of points to simulate, can vary if two points are too far from each other
    int m_simPointsDensity;

    /// ABCD matrix of the component (not used in v1.3)
    QVector<complexF> m_ABCDmatrix;
};

#endif // COMPONENT_H
