/****************************************************************************
 * QSmithTool :                                                             *
 *   Copyright (C) 2015 by Sébastien Lissarre                               *
 *   sebastien.lissarre@gmail.com                                           *
 *                                                                          *
 * Smith Chart drawing base :                                               *
 *   Copyright (C) 2008 by Lisandro Damián Nicanor Pérez Meyer              *
 *   perezmeyer@gmail.com                                                   *
 *                                                                          *
 *                                                                          *
 *  This file is part of QSmithTool.                                        *
 *                                                                          *
 *  QSmithTool is free software: you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation, either version 3 of the License, or       *
 *  (at your option) any later version.                                     *
 *                                                                          *
 *  QSmithTool is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with QSmithTool.  If not, see <http://www.gnu.org/licenses/>.     *
 ***************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QApplication>
#include <QMainWindow>
#include <QWidget>
#include <QVector>
#include <QPointF>
#include <QButtonGroup>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QHeaderView>
#include <QLabel>
#include <QLineEdit>
#include <QMenuBar>
#include <QPlainTextEdit>
#include <QPushButton>
#include <QSplitter>
#include <QStatusBar>
#include <QTabWidget>
#include <QString>
#include <QGroupBox>
#include <QRadioButton>
#include <QCheckBox>
#include <QSpacerItem>
#include <QListWidget>
#include <QComboBox>
#include <QShortcut>
#include <QDebug>
#include <QTranslator>
#include <QFile>
#include <QTextStream>
#include <QScrollArea>

#include <cmath>

#include "defines.h"
#include "circuit.h"
#include "circuitSimulation.h"
#include "smithchart.h"
#include "dialogPointsConfig.h"
#include "returnLossChart.h"

// The main window
class MainWindow : public QMainWindow
{
	Q_OBJECT

public:
    MainWindow();
    ~MainWindow();

public slots:
    /// Start the simulator when simulate button is clicked or data are modified
    void simulationSlot(SimQuality _simQuality = STANDARD_SIM);

    /// Slot called when start point is modified
    void singleStartPointLineEditSlot();
    /// Slot called when the start point lock button is clicked
    void singleStartPointsLockButtonSlot();
    void multipleStartPointsLockButtonSlot();
    /// Slot called when the start points are modified in the chart and it call for an update
    void startPointsUpdateFromChartSlot();
    /// Slot called when user select "single point" or "multiple points" radio button
    void startPointRBGroupSlot(int numButton);
    /// Slot called when the button to configure multi points if clicked
    void multipleStartPointsConfigButtonSlot();
    void displayReturnLossCheckBoxClickedSlot(bool checked);
    void displayReturnLossCheckBoxClosedSlot();

    /// Slots called when target point is modified
    void targetPointCheckBoxClickedSlot(bool checked);
    void targetPointLineEditSlot();
    void targetPointLockButtonSlot();
    void targetPointUpdateFromChartSlot();

    /// Slots called when default impedance on chart is modified
    void changeChartImpedanceCheckBoxSlot(bool checked);
    void changeChartImpedanceSlot();

    /// Save/open of circuit file
    void saveFileActionSlot();
    void saveFileAsActionSlot();
    void openFileActionSlot();
    /// Reset all the user interface and datas
    void clearAllSlot();

    /// Allow the user to force fast simulation a to start a very smooth render
    void forceFastSimActionSlot(bool _checked);
    void startSmoothRenderActionSlot();
    void displayQCirclesActionSlot(bool _checked);

    /// Update impedance, return loss, etc in status bar when mouse is moving
    void updateStatusBar();

    /// Update the display widget when a component is added or modified
    void updateCircuitDisplayWidget();
    /// Called when user double click in a component in list
    void doubleClickInCompListSlot(QModelIndex index);
    /// Called when user rigth click on the circuit display list
    void circuitListViewContextMenuRequested(const QPoint &);

    /// Language menu
    void languageSelectedSlot();

    /// About menu
    void shortcutHelpSlot();
    void aboutSlot();

    /// Save the config and circuit in the "backup" folder to undo action with Ctrl+Z
    void autoSave();
    void loadPreviousAutoSave();
    void clearAllAutoSaveFiles();

protected :
    void closeEvent ( QCloseEvent *event );
    void keyPressEvent( QKeyEvent *event );
    void wheelEvent ( QWheelEvent * event );

private:
    /// Draw the circuit in the second tab "Circuit display"
    void drawCircuitInDisplayWidget();

    /// Save the circuit & start points to a file
    void saveFile(QString _fileName);
    void openFile(QString _fileName);

    /// Translation
    int languageSelected;
    QTranslator *translator;

    /// List of electronic components in circuit
    Circuit *m_circuit;

    /// Smith chart
    SmithChart *m_smithChart;

    /// To open a window with the return loss chart when activated
    ReturnLossChart *m_returnLossChart;

    /// Internal variable to make sure there is always a correct value stored even if
    /// user enter a wrong value
    double m_chartImpedanceSelected;
    complexFAndFreq m_startPointSelected;

    /// Equal true if the fast simulation is forced
    bool m_forcedFastSimulation;

    /// Number of last autosave file
    int m_numLastAutosave;
    bool m_autoSaveEnable;

    /// User interface
    QWidget *centralWidget;
    QGridLayout *gridLayout;
    QSplitter *splitter;
    QTabWidget *chartToolBox;
    QWidget *dataPage;
    QWidget *circuitPage;
    QGridLayout *gridLayoutDataPage;
    QVBoxLayout *layoutCircuitPage;

    QButtonGroup *startPointRadioButtonGroup;

    /// Single point group box
    QRadioButton * singlePointRadioButton;
    QGroupBox *singlePointGroupBox;
    QLabel *singleStartPointImpedanceLabel;
    QLabel *singleStartPointRealImpedanceLabel;
    QLineEdit *singleStartPointRealImpedanceLineEdit;
    QLabel *singleStartPointImagImpedanceLabel;
    QLineEdit *singleStartPointImagImpedanceLineEdit;
    QPushButton *singleStartPointLockButton;
    QLabel *singleFreqLabel;
    QLineEdit *singleFreqLineEdit;
    QLabel *singleFreqUnitLabel;
    QHBoxLayout *singleStartPointLayoutH1, *singleStartPointLayoutH2;
    QVBoxLayout *singleStartPointLayoutV1;

    /// multi point group box
    QRadioButton * multiStartPointsRadioButton;
    QGroupBox *multiStartPointsGroupBox;
    QLabel *multiStartPointsLabelConfig;
    QPushButton *multiStartPointsConfigButton;
    QPushButton *multiStartPointsLockButton;
    QCheckBox *multiStartPointOnlyEndPointCheckBox;
    QCheckBox *displayReturnLossCheckBox;
    QHBoxLayout *multiStartPointsLayoutH1;
    QVBoxLayout *multiStartPointLayoutV1;

    /// Target point box
    QGroupBox *targetPointGroupBox;
    QLabel *targetPointImpedanceLabel;
    QLabel *targetPointRealImpedanceLabel;
    QLineEdit *targetPointRealImpedanceLineEdit;
    QLabel *targetPointImagImpedanceLabel;
    QLineEdit *targetPointImagImpedanceLineEdit;
    QPushButton *targetPointLockButton;
    QHBoxLayout *targetPointLayoutH1;

    /// Normalize impedance checkbox
    QGroupBox *chartImpedanceGroupBox;
    QLabel *chartImpedanceLabel;
    QLineEdit *chartImpedanceLineEdit;
    QLabel *chartImpedanceUnitLabel;
    QHBoxLayout *chartImpedanceLayoutH1;

    QSpacerItem *spacer;

    /// Menu bar actions
    QAction *saveFileAction;
    QAction *saveFileAsAction;
    QAction *openFileAction;
    QString *m_saveFileName;

    /// Circuit display page
    QListWidget *circuitListView;
    QAction *contextMenuRemoveAction;
    QAction *contextMenuInsertLINE_S;
    QAction *contextMenuInsertCAPA_P;

    /// Menubar
    QMenuBar *menubar;
    QAction *forceFastSimAction;
    QAction *startSmoothRenderAction;
    QAction *displayQCirclesAction;

    QAction *ENGAction;
    QAction *FRAction;
    QActionGroup *languageGroup;

    QStatusBar *statusbar;
};

#endif

