
#include "smithchart.h"

#define ZOOM_IN 1
#define ZOOM_OUT 0

#define PAINTER_LENGTH 1024
#define PAINTER_LENGTH_DIV_2 512
#define PAINTER_CTR_TO_ZERO_LENGTH PAINTER_LENGTH_DIV_2*0.875f // distance from chart center to 0+j0 on the chart
#define DIST_SELECT 15.0

#define NONE -1


SmithChart::SmithChart(double impedanceRef, Circuit &_circuit) : r_circuit(_circuit)
{
    pixmapChartBackgroundGenerated = false;
    m_painterInitialized = false;

    m_showTargetPoints = true;
    m_startPointsLocked=false;
    m_targetPointLocked=false;
    m_showQualityCircles=false;
    zoomScale=1.0;
    zoomTranslate = QPointF(0.0, 0.0);
    m_zoomButtonPressed = false;
    m_controlKeyPressed = false;
    m_helpTextOpacity = 0.0; // help text in bottom left corner
    m_startPointSelected = NONE;
    m_targetPointSelected = false;
    newZoomRect_p = QRectF(0.0, 0.0, width(), height());
    m_bezierCurvesMode = true;

    // Enable/disable the sweep mode (only end points showed in red)
    m_onlyEndPointsModeEnable = false;

    // When user use the '+' icon
    m_showOtherCompPicts = false;
    m_compPictPrevUnderMouse = NOT_DEF;
    m_mouseOverSuppPixmap = false;
    m_radiusAnimationRunning = false;
    m_hideKeyPressed = false;

    // We define pens
    thickPen = QPen(palette().windowText(), 0.75, Qt::SolidLine, Qt::FlatCap);
    thinPen = QPen(palette().windowText(), 0.25, Qt::SolidLine, Qt::FlatCap);
    textPen = QPen(palette().windowText(), 0.25);
    //wideTextPen = QPen(palette().foreground(), 0.25);
    dataPointsPen = QPen(QColor("blue"), 2.0, Qt::SolidLine, Qt::RoundCap);
    freePointsPen = QPen(QColor("green"), 10.0, Qt::SolidLine, Qt::RoundCap);
    lineDataPen = QPen(QColor("blue"), 1.0);

    posXYUnderMouse_w = QPointF(0.0,0.0);

    clearStartImpedancePoints();
    clearTargetImpedancePoint();

    setChartImpedance(impedanceRef);

    // Smith chart picture arcs calculation
    calculateSmithChartInsideArcs();
}


SmithChart::~SmithChart()
{
}

/* When the user press a key the Smith Chart */
void SmithChart::keyPressEvent( QKeyEvent *event )
{
    if (event->modifiers()==Qt::ControlModifier)
    {
        //qDebug() << "Control key pressed";
        m_controlKeyPressed=true;
        if ((event->key()==Qt::Key_Z ))
        {
            emit undoActionSignal();
        }
    }

    switch ( event->key() )
    {
        case Qt::Key_Up:
            //qDebug() << "Up key pressed";
            break;
        case Qt::Key_Down:
            //qDebug() << "Down key pressed";
            break;
        case Qt::Key_C:
            r_circuit.removePreviewComponent();
            if (event->modifiers()==Qt::ShiftModifier)
                r_circuit.addDefaultComponent(CAPA_S);
            else
                r_circuit.addDefaultComponent(CAPA_P);
            m_showOtherCompPicts=false;
            emit startSim(STANDARD_SIM);
            emit componentsListUpdated();
            emit autoSaveSignal();
            update();
            break;
    case Qt::Key_R:
        r_circuit.removePreviewComponent();
        if (event->modifiers()==Qt::ShiftModifier)
            r_circuit.addDefaultComponent(RESIST_S);
        else
            r_circuit.addDefaultComponent(RESIST_P);
        m_showOtherCompPicts=false;
        emit startSim(STANDARD_SIM);
        emit componentsListUpdated();
        emit autoSaveSignal();
        update();
        break;
    case Qt::Key_L:
        r_circuit.removePreviewComponent();
        if (event->modifiers()==Qt::ShiftModifier)
            r_circuit.addDefaultComponent(LINE_OPEN);
        else
            r_circuit.addDefaultComponent(LINE_S);
        m_showOtherCompPicts=false;
        emit startSim(STANDARD_SIM);
        emit componentsListUpdated();
        emit autoSaveSignal();
        update();
        break;
    case Qt::Key_I:
        r_circuit.removePreviewComponent();
        if (event->modifiers()==Qt::ShiftModifier)
            r_circuit.addDefaultComponent(SELF_S);
        else
            r_circuit.addDefaultComponent(SELF_P);
        m_showOtherCompPicts=false;
        emit startSim(STANDARD_SIM);
        emit componentsListUpdated();
        emit autoSaveSignal();
        update();
        break;
    case Qt::Key_H:
        m_hideKeyPressed=true;
        update();
        break;
    default:
            //qDebug() << event->key() << endl;
            break;
    }
}
/* When the user release a key on the Smith Chart */
void SmithChart::keyReleaseEvent( QKeyEvent *event )
{
    if (event->modifiers()!=Qt::ControlModifier)
    {
        //qDebug() << "Control key released";
        m_controlKeyPressed=false;
    }

    switch ( event->key() )
    {
        case Qt::Key_Up:
            //qDebug() << "Up key released";
            break;
        case Qt::Key_Down:
            //qDebug() << "Down key released";
            break;
    case Qt::Key_H:
        m_hideKeyPressed=false;
        update();
        break;
    default:
            //qDebug() << event->key() << endl;
            break;
    }
}

/* When the user click on the Smith Chart */
void SmithChart::mousePressEvent(QMouseEvent *event)
{
    if (m_painterInitialized) // avoid mouse events before the painter coordinates system is valid
    {
        QPointF tmpMousePos_w = event->localPos();
        QPointF tmpMousePos_p = widgetToPainterTransform(tmpMousePos_w);

        if (event->button() == Qt::LeftButton)
        {
            qDebug() << "Pos in widget=" << tmpMousePos_w << "   Pos in painter=" << tmpMousePos_p << "   ImpedanceNorm=" << complexNumberToQString(XYWidgetToZcalc(tmpMousePos_w)) ;

            // Manage if user mouse is on the "supp (+)" picture => display other comp when clicked
            int sizePixDiv2 = (int)((20.0+17.0/zoomScale)/2.0);
            if ((abs(tmpMousePos_p.x()-m_suppPictPos_p.x())<sizePixDiv2) && (abs(tmpMousePos_p.y()-m_suppPictPos_p.y())<sizePixDiv2)) // mouse in the '+' pixmap
            {
                if ((m_showOtherCompPicts==false)&&(m_startPointSelected==NONE)&&(m_hideKeyPressed==false))
                {
                    qDebug() << "User mouse on '+' pict";
                    m_showOtherCompPicts = true;
                    m_compPictPrevUnderMouse=NOT_DEF;

                    m_animIconRadius = new QVariantAnimation(this);
                    connect(m_animIconRadius, SIGNAL(valueChanged(QVariant)), this, SLOT(radiusAnimatorInternal(QVariant)));
                    double maxCirclePixRadius = 35.0+35.0/zoomScale;
                    m_animIconRadius->setStartValue(5);
                    m_animIconRadius->setEndValue((int)maxCirclePixRadius);
                    m_animIconRadius->setDuration(100);
                    m_animIconRadius->setEasingCurve(QEasingCurve::InOutQuint);
                    m_animIconRadius->start(QAbstractAnimation::DeleteWhenStopped);
                    m_radiusAnimationRunning=true;
                }
            }
            else {
                // Find a start point
                if ((m_showOtherCompPicts==false) && (m_startPointsLocked==false)) {
                    m_startPointSelected = findStartPointUnderMouse(tmpMousePos_w);
                }

                // Find a target point
                if ((m_showOtherCompPicts==false) && (m_targetPointLocked==false)) {
                    m_targetPointSelected = findTargetPointUnderMouse(tmpMousePos_w);
                }
            }
        }
        else if (event->button() == Qt::RightButton)
        {
            r_circuit.removeLastComponent();
            emit startSim(STANDARD_SIM);
            emit componentsListUpdated();
        }
        else if (event->button() == Qt::MiddleButton)
        {
            m_zoomButtonPressed = true;
            updateZoomRectangle(tmpMousePos_w.toPoint(),ZOOM_IN);
            update();
        }
    }
}

void SmithChart::mouseMoveEvent(QMouseEvent *event)
{
    static int lastItemModified = NONE;
    static QPointF oldMousepos = QPointF(0.0, 0.0);
    static QPointF deltaMousepos = QPointF(0.0, 0.0);

    if (m_painterInitialized) // avoid mouse events before the painter coordinates system is valid
    {
        QPointF tmpMousePos_w = event->localPos();
        QPointF tmpMousePos_p = widgetToPainterTransform(tmpMousePos_w);
        complexF tmpMousePos_Z = XYWidgetToZcalc(tmpMousePos_w);

        posXYUnderMouse_w = tmpMousePos_w;
        posZUnderMouse = tmpMousePos_Z;
        emit posUnderMouseUpdated();

        // Moving start point when user has left clicked on it
        if (m_startPointSelected != NONE)
        {
            enableBezierCurvesDrawing(false);
            moveStartPoint(m_startPointSelected, tmpMousePos_Z*m_chartImpedance);
            if (m_startPointSelected==0) { // first start point selected
                emit startPointsUpdated(); // signal to update the window display for start point
            }
            emit startSim(QUICK_SIM);
            oldMousepos = tmpMousePos_w;
        }

        // Moving target point when user has left clicked on it
        if (m_targetPointSelected)
        {
            setTargetPoint(tmpMousePos_Z*m_chartImpedance);
            emit targetPointUpdated(); // signal to update the window display for target point

            oldMousepos = tmpMousePos_w;
        }


        // Find if a data point is under the mouse
        if (m_showOtherCompPicts==false) // only if the user is not choosing a new comp
        {
            int numItem = findDataPointUnderMouse(tmpMousePos_w, DIST_SELECT*0.4+(DIST_SELECT*0.6)*zoomScale); // if no zoom => =DIST_SELECT
            if (numItem!=NONE)//&&(numItem!=START_POINT)&&(numItem!=TARGET_POINT))
            {
                if (numItem != lastItemModified)
                {
                    lastItemModified=numItem;
                    r_circuit.resetAllComponentsMouseOverState();
                    r_circuit.setComponentMouseOverState(numItem, true);

                    // Fade in help text in bottom left corner (tip for "ctrl key")
                    //if (m_animTextOpacity->state() == QAbstractAnimation::Running)
                    //    m_animTextOpacity->stop();
                    m_textOpacityAnimator = new QVariantAnimation(this);
                    connect(m_textOpacityAnimator, SIGNAL(valueChanged(QVariant)), this, SLOT(textOpacityAnimatorFct(QVariant)));
                    QVariant valStart =  QVariant(m_helpTextOpacity);
                    m_textOpacityAnimator->setStartValue(valStart);
                    m_textOpacityAnimator->setEndValue(1.0);
                    m_textOpacityAnimator->setDuration(600);
                    m_textOpacityAnimator->setEasingCurve(QEasingCurve::Linear);
                    m_textOpacityAnimator->start(QAbstractAnimation::DeleteWhenStopped);
                    //qDebug() << "Anim fade in start"; //, from " << valStart << "to " << 1.0;

                    update();
                }
            }
            else if (numItem==NONE)
            {
                if (lastItemModified!=NONE)
                {
                    lastItemModified=NONE;
                    r_circuit.resetAllComponentsMouseOverState();

                    m_textOpacityAnimator = new QVariantAnimation(this);
                    connect(m_textOpacityAnimator, SIGNAL(valueChanged(QVariant)), this, SLOT(textOpacityAnimatorFct(QVariant)));
                    m_textOpacityAnimator->setStartValue(m_helpTextOpacity);
                    m_textOpacityAnimator->setEndValue(0.0);
                    m_textOpacityAnimator->setDuration(600);
                    m_textOpacityAnimator->setEasingCurve(QEasingCurve::Linear);
                    m_textOpacityAnimator->start(QAbstractAnimation::DeleteWhenStopped);
                    //qDebug() << "Anim fade out start"; //, from " << m_helpTextOpacity << "to " << 0.0;
                }
            }
        }

        // Manage the screen translation when user want to move the view
        if (m_zoomButtonPressed == true)
        {
            deltaMousepos = tmpMousePos_w - oldMousepos;
            oldMousepos = tmpMousePos_w;
            zoomTranslate = zoomTranslate + deltaMousepos/(zoomScale/2.0);
            //qDebug() << "Zoom translate : " << zoomTranslate;

            pixmapChartBackgroundGenerated=false; // chart background pixmap need to be updated
            update();
        }
        oldMousepos = tmpMousePos_w;

        // Manage if user has pressed left click on the "supp (+)" picture, and then move the mouse to the icons in circle
        if ((m_showOtherCompPicts==true)&&(m_radiusAnimationRunning==false)&&(m_hideKeyPressed==false))
        {
            // Mouse on a component icon? => add this component
            ComponentType tmpCompType = findIconUnderMouse(tmpMousePos_p);
            //qDebug() << tmpCompType;
            if (r_circuit.getPreviewComponentType()!=tmpCompType)
            {
                qDebug() << "Mouse on " << tmpCompType;
                r_circuit.setPreviewComponent(tmpCompType); // if tmpCompType==NOT_DEF, remove the preview
                m_compPictPrevUnderMouse=tmpCompType;
                emit startSim(STANDARD_SIM);
                emit componentsListUpdated();
                update();
            }
        }
        else
        {
            if ((m_showOtherCompPicts==false)&&(m_startPointSelected==NONE)&&(m_hideKeyPressed==false)) // if mouse is only over '+' pixmap => make it brigther
            {
                int sizePixDiv2 = (int)((20.0+17.0/zoomScale)/2.0);
                //qDebug() << abs(tmpMousePos_p.x()-m_suppPictPos_p.x());
                if ((abs(tmpMousePos_p.x()-m_suppPictPos_p.x())<sizePixDiv2) && (abs(tmpMousePos_p.y()-m_suppPictPos_p.y())<sizePixDiv2)) // mouse in the '+' pixmap
                {
                    if (m_mouseOverSuppPixmap==false) {
                        m_mouseOverSuppPixmap = true;
                        update();
                    }
                    else
                        m_mouseOverSuppPixmap = true; // no update here to avoid slowing the app with constant window update

                    if (QApplication::overrideCursor()!=0) // no cursor defined
                        QApplication::restoreOverrideCursor();
                }
                else // if mouse not on '+' icon
                {
                    if (m_mouseOverSuppPixmap==true) {
                        m_mouseOverSuppPixmap = false;
                        update();
                    }
                    else {
                        m_mouseOverSuppPixmap = false; // no update here to avoid slowing the app with constant window update
                    }

                    // Change mouse icon when the mouse is over a start point
                    if (((findStartPointUnderMouse(tmpMousePos_w) != NONE) || (findTargetPointUnderMouse(tmpMousePos_w) != false)) && (m_showOtherCompPicts==false) && (m_startPointsLocked==false))
                    {
                        if (QApplication::overrideCursor()==0)
                            QApplication::setOverrideCursor(QCursor(Qt::SizeAllCursor));
                    }
                    else // not on a start point, restore mouse cursor
                    {
                        if (QApplication::overrideCursor()!=0) // no cursor defined
                            QApplication::restoreOverrideCursor();
                    }
                }
            }
        }
    }
}

void SmithChart::mouseReleaseEvent(QMouseEvent *event)
{
    if (m_painterInitialized) // avoid mouse events before the painter coordinates system is valid
    {
        QPointF tmpMousePos_w = event->localPos();
        QPointF tmpMousePos_p = widgetToPainterTransform(tmpMousePos_w);

        if (event->button() == Qt::LeftButton)
        {
            // Manage if user realease the left click on a component icon or not => add this component or remove all preview icons
            if (m_showOtherCompPicts==true)
            {
                // Is a component to add under mouse?
                ComponentType tmpCompType = findIconUnderMouse(tmpMousePos_p);
                if ((tmpCompType!=NOT_DEF) && (m_radiusAnimationRunning==false)) // release on a component preview => add this component
                {
                    r_circuit.removePreviewComponent();
                    r_circuit.addDefaultComponent(tmpCompType);
                    m_showOtherCompPicts=false;
                    emit startSim(STANDARD_SIM);
                    emit componentsListUpdated();
                    emit autoSaveSignal();
                    update();
                    qDebug() << "Move cursor!";
                    complexF lastPoint = r_circuit.getLastPointNotInPreview();
                    QPointF myPointF_w = painterToWidgetTransform(ZtoXYPainterCalc(lastPoint)+QPointF(20.0,-30.0));
                    QPoint center = mapToGlobal(myPointF_w.toPoint());
                    qDebug() << "Target pos : " << center << " | in parent  : " << mapToParent(myPointF_w.toPoint()) << "  | parentGeometry " << geometry();
                    if ((mapToParent(myPointF_w.toPoint()).x()>0) && (mapToParent(myPointF_w.toPoint()).y()>0)  && (mapToParent(myPointF_w.toPoint()).x()< geometry().width())  && (mapToParent(myPointF_w.toPoint()).y()<height()))
                        QCursor::setPos(center);
                }
                else // mouse released nowhere => only hide all the preview icons
                {
                    r_circuit.removePreviewComponent();
                    m_showOtherCompPicts=false;
                    emit startSim(STANDARD_SIM);
                    emit componentsListUpdated();
                    update();
                }
            }

            // When mouse is released after moving s start point,
            // the last simulation has been made with just few points,
            // so we have to simulate a last time with full points
            if (m_startPointSelected != NONE)
            {
                emit startSim(STANDARD_SIM);
                emit componentsListUpdated();
                update();
                emit autoSaveSignal();
            }
            // End of start point moving
            m_startPointSelected=NONE;
            enableBezierCurvesDrawing(true); // reactivate smooth render

            // End of target point moving
            m_targetPointSelected=false;
        }
        else if (event->button() == Qt::MiddleButton)
        {
            m_zoomButtonPressed = false;
            update();
        }
    }
}


void SmithChart::wheelEvent ( QWheelEvent * event )
{
    int scrollValue = QPointF(event->angleDelta() / 120).y();
    double oldScale = zoomScale;

    // user scroll over an empty place : Zoom+ & Zoom-
    //if ((findDataPointUnderMouse(event->pos(), DIST_SELECT_WIDE) == NONE) && (!m_controlKeyPressed)) // no point found under mouse => zoom
    if (!m_controlKeyPressed) // Control key not pressed => zoom
    {
        if (scrollValue>0)
        {
            if (zoomScale<32.0)
                updateZoomRectangle(event->pos(),ZOOM_IN);
            qDebug() << "Zoom IN";
        }
        else
        {
            if (zoomScale>=0.9)
                updateZoomRectangle(event->pos(),ZOOM_OUT);
            qDebug() << "Zoom OUT";
        }

        QRectF oldZoomRect_p = widgetToPainterTransform(QRectF(0,0, width(), height()));

        if (newZoomRect_p.height() < newZoomRect_p.width())
            zoomScale = zoomScale * (oldZoomRect_p.height() / newZoomRect_p.height());
        else
            zoomScale = zoomScale * (oldZoomRect_p.width() / newZoomRect_p.width());
        zoomTranslate = zoomTranslate + (oldZoomRect_p.center() - newZoomRect_p.center());

        //qDebug() << "Old rect_p : " << oldZoomRect_p << "   New rect_p : " << newZoomRect_p;
        qDebug() << "Old scaling : " << oldScale << "   New scaling : " << zoomScale;

        if (zoomScale<=1.0)
        {
            qDebug() << "Recentering";
            zoomTranslate = QPointF(0.0, 0.0);
        }
    }
    else // the user scroll over a point => increase or decrease comp value under the mouse
    {
        for (int i=0 ; i<r_circuit.getSize() ; i++)
        {
            if (r_circuit.getComponentMouseOverState(i))
            {
                double tmpRatioPos;
                ComponentType tmpType = r_circuit.getComponentType(i);
                if ((tmpType==CAPA_S)||(tmpType==SELF_P)||(tmpType==SELF_P)||(tmpType==RESIST_P))
                    tmpRatioPos = 0.9;
                else
                    tmpRatioPos = 1.1;

                if (scrollValue>0)
                    r_circuit.ratioOnCompFirstParameter(i, tmpRatioPos);
                else
                    r_circuit.ratioOnCompFirstParameter(i, 1.0/tmpRatioPos);

                emit componentsListUpdated();
                emit startSim(STANDARD_SIM);
                emit autoSaveSignal();
            }
        }
    }

    pixmapChartBackgroundGenerated=false; // chart background pixmap need to be updated
    update(); // paintEvent

}


void SmithChart::mouseDoubleClickEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton)
    {
        int numDataPointUnderMouse=findDataPointUnderMouse(event->localPos(), DIST_SELECT*0.75+(DIST_SELECT*0.25)/zoomScale); // if no zoom => =DIST_SELECT

        // the user double click on component : dialog to modify parameter
        if (numDataPointUnderMouse != NONE)//&&(numItemUnderMouse != START_POINT)&&(numItemUnderMouse != TARGET_POINT))
        {
            componentDoubleClick(numDataPointUnderMouse);
        }
    }
}


void SmithChart::componentDoubleClick(int numComponent)
{
    ComponentType type = r_circuit.getComponentType(numComponent);
    CompParametersList listParams = r_circuit.getComponentParametersDenorm(numComponent, m_chartImpedance);

    if ((type == LINE_S) || (type == LINE_OPEN) || (type == LINE_SHORT))
    {
        dialogLine = new dialogParametersLine(this, listParams);
        connect(dialogLine, SIGNAL(setPermitivityToAllLinesClicked(double)), this, SLOT(setPermitivityToAllLines(double)));
        if (dialogLine->exec() == QDialog::Accepted)
        {
            QVector<double> &returnVal = dialogLine->getValues();
            for (int i=0 ; i<returnVal.size() ; i++)
            {
                r_circuit.setComponentParameterDenorm(numComponent, i, returnVal[i], m_chartImpedance);
            }
        }
        else
            qDebug() << "User cancellation";
    }
    else
    {
        dialogComp = new dialogParametersDiscrete(this, type, listParams);
        if (dialogComp->exec() == QDialog::Accepted)
        {
            QVector<double> &returnVal = dialogComp->getValues();
            for (int i=0 ; i<returnVal.size() ; i++)
            {
                r_circuit.setComponentParameterDenorm(numComponent, i, returnVal[i], m_chartImpedance);
            }
        }
        else
            qDebug() << "User cancellation";
    }

    emit componentsListUpdated();
    emit startSim(STANDARD_SIM);
    emit autoSaveSignal();
}


void SmithChart::updateZoomRectangle(QPoint posMouse, bool zoomInOut)
{
    posXYUnderMouse_w = posMouse;
    posZUnderMouse = XYWidgetToZcalc(posXYUnderMouse_w);

    QRectF newZoomRect_w;

    double sizeRectW, sizeRectH;
    double ratioWidthMouse = posXYUnderMouse_w.x()/width();
    double ratioHeightMouse = posXYUnderMouse_w.y()/height();

    if (zoomInOut == ZOOM_IN)
    {
        sizeRectW = width()*0.5;
        sizeRectH = height()*0.5;
        // Zoom rectangle = area around the mouse with position ratio
        newZoomRect_w.setTopLeft(posXYUnderMouse_w - QPointF(sizeRectW*ratioWidthMouse, sizeRectH*ratioHeightMouse));
        newZoomRect_w.setBottomRight(posXYUnderMouse_w + QPointF(sizeRectW*(1.0-ratioWidthMouse), sizeRectH*(1.0-ratioHeightMouse)));
    }
    else
    {
        sizeRectW = width()*2.0;
        sizeRectH = height()*2.0;
        // Zoom rectangle = area around the mouse with position ratio
        newZoomRect_w.setTopLeft(posXYUnderMouse_w - QPointF(sizeRectW*ratioWidthMouse, sizeRectH*ratioHeightMouse));
        newZoomRect_w.setBottomRight(posXYUnderMouse_w + QPointF(sizeRectW*(1.0-ratioWidthMouse), sizeRectH*(1.0-ratioHeightMouse)));
    }

    newZoomRect_p.setTopLeft(widgetToPainterTransform(newZoomRect_w.topLeft()));
    newZoomRect_p.setBottomRight(widgetToPainterTransform(newZoomRect_w.bottomRight()));
    //qDebug() << "old zoom rect_w=" << geometry().topLeft() << geometry().bottomRight() << "   old rect_p =" << widgetToPainterTransform(QRectF(geometry())).topLeft() << widgetToPainterTransform(QRectF(geometry())).bottomRight();
    //qDebug() << "new zoom rect_w=" << newZoomRect_w.topLeft() << newZoomRect_w.bottomRight() << "   new rect_p =" << newZoomRect_p.topLeft() << newZoomRect_p.bottomRight()  ;

    return;
}

complexF SmithChart::getLastZPosMouse()
{
    return posZUnderMouse;
}

QPointF SmithChart::getLastXYPosMouse()
{
    return posXYUnderMouse_w;
}


void SmithChart::clearTargetImpedancePoint()
{
    targetImpedancePoint = complexF(1.0 , 0.0); // 50 ohm

    qDebug() << "Chart target point cleaned";
    update(); // paintEvent
}

void SmithChart::clearStartImpedancePoints()
{
    startImpedancesPoints.clear();

    qDebug() << "All chart start data points cleaned";
    update(); // paintEvent
}

double SmithChart::getChartImpedance()
{
    return m_chartImpedance;
}


void SmithChart::setChartImpedance(double impedance)
{
    m_chartImpedance=impedance;
    pixmapChartBackgroundGenerated=false; // redraw the chart
    update(); // paintEvent
}


void SmithChart::setStartPoints(const QVector<complexFAndFreq> _pointVector) // not normalized (reference : m_chartImpedance)
{
    // Dimensioning array
    startImpedancesPoints.clear();
    startImpedancesPoints.resize(_pointVector.size());

    // Copy
    for(int i=0 ; i < _pointVector.size() ; i++)
    {
        startImpedancesPoints[i].point=_pointVector[i].point/m_chartImpedance;
        startImpedancesPoints[i].freq=_pointVector[i].freq;
        qDebug() << "Set start point (ref " << m_chartImpedance << "ohm) :" << complexNumberToQString(_pointVector[i].point) << "," << _pointVector[i].freq;
    }
    update(); // paintEvent
}
void SmithChart::setStartPointsNormalized(const QVector<complexFAndFreq> _pointVector) // normalized
{
    // Dimensioning array
    startImpedancesPoints.clear();
    startImpedancesPoints.resize(_pointVector.size());

    // Copy
    for(int i=0 ; i < _pointVector.size() ; i++)
    {
        startImpedancesPoints[i].point=_pointVector[i].point;
        startImpedancesPoints[i].freq=_pointVector[i].freq;
        qDebug() << "Set start point (normalized) :" << complexNumberToQString(_pointVector[i].point) << "," << _pointVector[i].freq;
    }
    update(); // paintEvent
}
void SmithChart::moveStartPoint(int _numStartPoint, const complexF _newPoint)
{
    startImpedancesPoints[_numStartPoint].point=_newPoint/m_chartImpedance;
    startImpedancesPoints[_numStartPoint].freq=startImpedancesPoints[0].freq;
    qDebug() << "Moved start point[" << _numStartPoint << "] :" << complexNumberToQString( startImpedancesPoints[_numStartPoint].point) << "," <<  startImpedancesPoints[_numStartPoint].freq;
    update(); // paintEvent
}


QVector<complexFAndFreq> & SmithChart::getStartPointsNorm()
{
    return startImpedancesPoints;
}


complexF SmithChart::getFirstStartPointDenorm()
{
    if (startImpedancesPoints.size()>0)
        return startImpedancesPoints.first().point*m_chartImpedance;
    else
    {
        qDebug() << "Error in getFirstStartPointDenorm(), vector empty";
        return complexF(0.0,0.0);
    }
}


void SmithChart::setTargetPoint(const complexF _newPoint)
{
    targetImpedancePoint=_newPoint/m_chartImpedance;
    qDebug() << "Set target point (ref " << m_chartImpedance << "ohm) :" << complexNumberToQString(_newPoint);
    update(); // paintEvent
}
void SmithChart::setTargetPointNormalized(const complexF _newPointNorm)
{
    targetImpedancePoint=_newPointNorm;
    qDebug() << "Set target point (normalized) :" << complexNumberToQString(targetImpedancePoint);
    update(); // paintEvent
}
complexF SmithChart::getTargetPoint()
{
    return targetImpedancePoint;
}


void SmithChart::enableTargetPoints(bool _enable)
{
    m_showTargetPoints = _enable;
    update(); // paintEvent
}

void SmithChart::enableQCirclesDrawing(bool _enable)
{
    m_showQualityCircles = _enable;
    pixmapChartBackgroundGenerated=false; // redraw the chart
    update(); // paintEvent

}

void SmithChart::enableBezierCurvesDrawing(bool _enable)
{
    m_bezierCurvesMode = _enable;
}

void SmithChart::setPermitivityToAllLines(double relPerm)
{
    ComponentType compType;

    for(int i=0 ; i < r_circuit.getSizeWithoutPreviewed() ; i++)
    {
        compType = r_circuit.getComponentType(i);
        if ((compType==LINE_S)||(compType==LINE_OPEN)||(compType==LINE_SHORT))
        {
            qDebug() << "Change permitivity of comp n°" << i;
            r_circuit.setComponentParameterDenorm(i, 2, relPerm, m_chartImpedance);
        }
    }
    emit startSim(STANDARD_SIM);
    emit componentsListUpdated();
    update();
}

void SmithChart::paintEvent(QPaintEvent * /* the event */)
{
    //qDebug() << "Paint event()";
    //qDebug() << "Paint event :  Scale=" << zoomScale << "  Translate=" << zoomTranslate;

    //QElapsedTimer timerElaps;
    //qint64 ns_elaps;
    //timerElaps.start();

    painter = new QPainter(this);
    m_painterInitialized=true;
    painter->setRenderHints(QPainter::Antialiasing, true);
    painter->setRenderHints(QPainter::TextAntialiasing, true);
    painter->setRenderHints(QPainter::SmoothPixmapTransform, true);
    painter->fillRect(0, 0, width(), height(), QBrush(QColor(255, 255, 255)));

    int sidePainter_w = qMin(width(), height());
    //qDebug() << "w=" << width() << "  h=" << height() << "   min side=" << sidePainter_w;

    //painter.setViewport(0, 0, side, side); // this one is ok but the chart is stuck at the left of the screen
    painter->setViewport((width()-sidePainter_w)/2, (height()-sidePainter_w)/2, sidePainter_w, sidePainter_w); // centrage de la vue
    painter->setWindow(-PAINTER_LENGTH_DIV_2, -PAINTER_LENGTH_DIV_2, PAINTER_LENGTH, PAINTER_LENGTH);

    if (pixmapChartBackgroundGenerated==false) // if the background as changed (zoom or translate)
        generatePixmapChartBackground(); // redraw it

    painter->setViewTransformEnabled(false);
    painter->drawPixmap(0, 0, pixmapChartBackground); // draw the chart pixmap at (0,0) to fill the entire widget (generated just before with the good size)
    painter->setViewTransformEnabled(true);

    //qDebug() << "DrawChart calculation time :" << ns_elaps/1000000 << "ms";

    // Help text for tips in bottom left corner
    painter->save();
    painter->setOpacity(m_helpTextOpacity);
    painter->setBrush(QBrush(Qt::black));
    painter->setFont(QFont("monospace", 14, QFont::Normal));
    painter->drawText(QPoint(-680, 490),tr("Use \"Ctrl\" key to modify value with mouse wheel")); // text at down left
    painter->restore();

    // Scale and translate for zoom
    painter->scale(zoomScale,zoomScale);
    painter->translate(zoomTranslate);

    drawDataPoints(painter);
    if (m_hideKeyPressed==false) // 'space'H' key allow user to hide the '+' icon
        drawAddCompPixmap(painter);

    //ns_elaps = timerElaps.nsecsElapsed();
    //qDebug() << "End of paintevent(), painting time : " << ns_elaps/1000000.0 << "ms";
    //qDebug() << "End of paintevent()";
}

void SmithChart::resizeEvent( QResizeEvent *)
{
    pixmapChartBackgroundGenerated=false; // need to regenerate the chart background
}

void SmithChart::generatePixmapChartBackground()
{
    qDebug() << "Chart background pixmap generation";

    int sidePainter_w = qMin(width(), height());
    pixmapChartBackground = QPixmap(width(), height());
    pixmapChartBackground.fill(Qt::white);
    QPainter painterBackground(&pixmapChartBackground);
    painterBackground.setRenderHints(QPainter::Antialiasing, true);
    painterBackground.setRenderHints(QPainter::TextAntialiasing, true);
    painterBackground.setRenderHints(QPainter::SmoothPixmapTransform, true);
    painterBackground.setViewport((width()-sidePainter_w)/2, (height()-sidePainter_w)/2, sidePainter_w, sidePainter_w); // centrage de la vue
    painterBackground.setWindow(-PAINTER_LENGTH_DIV_2, -PAINTER_LENGTH_DIV_2, PAINTER_LENGTH, PAINTER_LENGTH);
    painterBackground.scale(zoomScale,zoomScale);
    painterBackground.translate(zoomTranslate);
    drawChart(&painterBackground);
    painterBackground.end();
    //pixmapChartBackground->save(QString("./chartBackgroundHD.png"), "png", 100);

    pixmapChartBackgroundGenerated=true;
}

QPointF SmithChart::ZtoXYPainterCalc(const complexF &pointZ)
{
    double real=pointZ.real();
    double imaginary=pointZ.imag();
    double x = ((real*real)+(imaginary*imaginary)-1.0);
    x = x / ((real+1.0)*(real+1.0)+(imaginary*imaginary));
    x = x * (double)(PAINTER_CTR_TO_ZERO_LENGTH);
    /*
    Note that the y coordinate gets inverted, that is because the graphic
    coordinates are inverted.
    */
    double y = 2.0*imaginary/((real+1.0)*(real+1.0)+(imaginary*imaginary));
    y = -y * (double)(PAINTER_CTR_TO_ZERO_LENGTH);

    return QPointF(x, y);
}

complexF SmithChart::XYWidgetToZcalc(const QPointF &pointXY_w)
{
    QPointF coordInPainter = widgetToPainterTransform(pointXY_w) / (double)(PAINTER_CTR_TO_ZERO_LENGTH);
    double xr = coordInPainter.x();
    double yr = -coordInPainter.y(); //  '-' because the y axis is inverted

    //qDebug() << "Pos in widget=" << pointXY << "  Pos in painter=" << coordInPainter;

    double real = 1.0 - (xr*xr) - (yr*yr);
    real = real / ((1.0-xr)*(1.0-xr) + (yr*yr));

    double imag = 2.0*yr / ((1.0-xr)*(1.0-xr) + (yr*yr));

    return complexF(real,imag);
}

QPointF SmithChart::widgetToPainterTransform(QPointF point)
{
    /*QTransform trMatrix = painter->combinedTransform();
    if (trMatrix.isInvertible())
    {*/
        QTransform matrixWidgetToPainter = painter->combinedTransform().inverted();
        return QPointF(matrixWidgetToPainter.map(point));
    /*}
    else
    {
        qDebug() << "widgetToPainterTransform() : matrix not invertible";
        return QPointF(0.0,0.0);
    }*/
}

QPointF SmithChart::painterToWidgetTransform(QPointF point)
{
    QTransform matrixPainterToWidget = painter->combinedTransform();

    return QPointF(matrixPainterToWidget.map(point));
}

QRectF SmithChart::widgetToPainterTransform(QRectF rect)
{
    QTransform matrixWidgetToPainter = painter->combinedTransform().inverted();

    return QRectF(matrixWidgetToPainter.mapRect(rect));
}

QRectF SmithChart::painterToWidgetTransform(QRectF rect)
{
    QTransform matrixPainterToWidget = painter->combinedTransform();

    return QRectF(matrixPainterToWidget.mapRect(rect));
}


void SmithChart::addArc3pointsToPath(QPen pen, QPointF a, QPointF b, QPointF c)
{
    double res_startAngle, res_sweepAngle;
    QRectF res_rectangle;

    arc3pointsCalc(a, b, c, &res_rectangle, &res_startAngle, &res_sweepAngle);

    if(pen == thinPen)
    {
        thinArcsPath.moveTo(a);
        thinArcsPath.arcTo(res_rectangle, (180.0-res_startAngle), -res_sweepAngle); // 0° at 3 o'clock, counter clockwise
    }
    else
    {
        thickArcsPath.moveTo(a);
        thickArcsPath.arcTo(res_rectangle, (180.0-res_startAngle), -res_sweepAngle); // 0° at 3 o'clock, counter clockwise
    }
}

void SmithChart::drawArc3points(QPainter *painter, QPointF a, QPointF b, QPointF c)
{
    double res_startAngle, res_sweepAngle;
    QRectF res_rectangle;

    arc3pointsCalc(a, b, c, &res_rectangle, &res_startAngle, &res_sweepAngle);
    painter->drawArc(res_rectangle, (180.0-res_startAngle)*16.0, -res_sweepAngle*16); // 0° at 3 o'clock, counter clockwise
}


void SmithChart::arc3pointsCalc(QPointF a, QPointF b, QPointF c, QRectF *_resRect, double *_resStartAngle, double *_resSweepAngle)
{
    double d = 2 * (a.x() - c.x()) * (c.y() - b.y()) + 2 * (b.x() - c.x()) * (a.y() - c.y());
    double m1 = (pow(a.x(), 2) - pow(c.x(), 2) + pow(a.y(), 2) - pow(c.y(), 2));
    double m2 = (pow(c.x(), 2) - pow(b.x(), 2) + pow(c.y(), 2) - pow(b.y(), 2));
    double nx = m1 * (c.y() - b.y()) + m2 * (c.y() - a.y());
    double ny = m1 * (b.x() - c.x()) + m2 * (a.x() - c.x());
    double cx = nx / d;
    double cy = ny / d;
    double dx = cx - a.x();
    double dy = cy - a.y();
    double distance = sqrt(dx * dx + dy * dy);
    QPointF va (a.x() - cx, a.y() - cy);
    QPointF vb (b.x() - cx, b.y() - cy);
    QPointF vc (c.x() - cx, c.y() - cy);
    QPointF xaxis (-1.0, 0.0);

    *_resStartAngle = (double)angleBetween(xaxis, va);
    *_resSweepAngle = (double)(angleBetween(va, vb) + angleBetween(vb, vc));
    *_resRect = QRectF((double)(cx - distance), (double)(cy - distance),
                     (double)(distance * 2), (double)(distance * 2));
}


double SmithChart::angleBetween(QPointF pt1, QPointF pt2) {
    double dot = pt1.x()*pt2.x() + pt1.y()*pt2.y(); // dot product
    double det = pt1.x()*pt2.y() - pt1.y()*pt2.x(); // determinant
    double angle = atan2(det, dot);  // atan2(y, x) or atan2(sin, cos);
    return (angle * 180 / _PI);
}

void SmithChart::drawChart(QPainter * painter)
{
    //qDebug() << "DrawChart()";

    // We set the pen and brush for the outern circles
    painter->setPen(thickPen);
    painter->setBrush(palette().foreground());

    // The outer circle
    QRectF rectangle(-PAINTER_LENGTH_DIV_2, -PAINTER_LENGTH_DIV_2, PAINTER_LENGTH, PAINTER_LENGTH);
    painter->drawArc(rectangle, 0, 5760);

    // Inner thin circle, wavelengths toward generator
    rectangle.adjust(16, 16, -16, -16);
    painter->setPen(thinPen);
    painter->drawArc(rectangle, 0, 5760);

    // Inner thick circle, wavelengths toward load
    rectangle.adjust(16, 16, -16, -16);
    painter->setPen(thickPen);
    painter->drawArc(rectangle, 0, 5760);

    // Inner thin circle, angle of reflection coefficient in degrees
    rectangle.adjust(16, 16, -16, -16);
    painter->setPen(thinPen);
    painter->drawArc(rectangle, 0, 5760);

    // Inner thick circle, angle of transmission coefficient in degrees
    rectangle.adjust(16, 16, -16, -16);
    painter->setPen(thickPen);
    painter->drawArc(rectangle, 0, 5760);


    /*
        The ticks in the wavelengths scales
        The scales advance in 0.002 degrees.
        0.5 [degrees] / 0.002 [degrees/tick] = 250 [tick]
        360º / 250 [tick] = 1.44º/[tick]
        So, i has to advance while i < 250
        Thick marcs: 0.002 * 5 = 0.01 (each time i % 5 == 0)
    */
    for(int i=0; i< 250; i++)
    {
        // We keep the transformation matrix
        painter->save();
        painter->rotate(-i*1.44);

        if(i % 5 == 0)
        {
            painter->setPen(thickPen);
            painter->drawLine(493, 0, 499, 0);
        }
        else
        {
            painter->setPen(thinPen);
            painter->drawLine(493, 0, 499, 0);
        }
        painter->restore();
    }

    /*
        Ticks in the angle of reflection coefficient scale
        This scale goes (-180º,180º], in steps of two degrees.
        So for a half circle we have:
        180º/2[º/tick] = 90 [tick]
    */
    for(int i=90; i > -90; i--)
    {
        // We keep the transformation matrix
        painter->save();
        painter->rotate(-i*2.0);

        if(i % 5 == 0)
        {
            painter->setPen(thickPen);
            painter->drawLine(QPointF(464.4, 0.0), QPointF(468, 0));
        }
        else
        {
            painter->setPen(thinPen);
            painter->drawLine(QPointF(464.2, 0.0), QPointF(468, 0));
        }
        painter->restore();
    }

    /*
        The numbers in the wavelengths scales
        This time there are only:
        0.5 [degrees] / 0.01 [degrees/tick] = 50 [tick]
        360º / 50 [tick] = 7.2 [º/tick]
    */

    // We set the font for the scale
    painter->setFont(QFont("monospace", 8, QFont::Light));
    painter->setPen(textPen);

    for(int i=0; i< 50; i++)
    {
        // We keep the transformation matrix
        painter->save();
        painter->rotate(-90);
        painter->rotate((-i*7.2));

        // Wavelengths towards generator scale
        if(i == 0)
        {
            painter->drawText(-14,-595,100,100,
                              Qt::AlignLeft | Qt::AlignBottom,
                              QString::number(0.0,'f',2));
        }
        else
        {
            painter->drawText(-14,-595,100,100,
                              Qt::AlignLeft | Qt::AlignBottom,
                              QString::number(0.5-(i*0.01),'f',2));
        }
        // Wavelengths towards load scale
        painter->drawText(-14,-578,100,100,
                          Qt::AlignLeft | Qt::AlignBottom,
                          QString::number(i*0.01,'f',2));

        painter->restore();
    }

    /*
        The numbers in the angle of reflection coefficient scale
        This scale goes (-180º,180º], in steps of 10 degrees.
        So for a half circle we have:
        180º/10[º/tick] = 18 [tick]
    */
    for(int i=18; i > -18; i--)
    {
        // We keep the transformation matrix
        painter->save();
        painter->rotate(90);
        painter->rotate((-i*10.0));

        if(i >= 0)
        {
            // From 180º to 0º
            // The formula in X centers the text in the ticks
            painter->drawText((int)((-4.0/9.0)*i-4),-565,100,100,
                              Qt::AlignLeft | Qt::AlignBottom,
                              QString::number(i*10));
        }
        else
        {
            // From 0º to -180º
            // The formula in X centers the text in the ticks
            painter->drawText((int)((4.0/9.0)*i-10),-565,100,100,
                              Qt::AlignLeft | Qt::AlignBottom,
                              QString::number(i*10));
        }
        painter->restore();
    }

    /*
        Ticks in the angle of transmission coefficient scale
        This scale goes [0,90], in steps of 1.
        This scale is the degrees that has the line that goes across the point
        Z = 0 + j0 and the intersection of the scale between the X axis.
    */
    /*
    static const float angleOfTransmissionR[] = {912.0, 911.5, 911.0, 910.5, 910.0, 909.0, 907.5, 905.5, 903.5, 901.5,
                                                 899.0, 896.0, 892.5, 889.5, 885.5, 882.5, 878.0, 874.0, 869.0, 863.5,
                                                 858.5, 853.5, 848.0, 842.0, 835.5, 829.5, 823.0, 816.0, 809.0, 802.0,
                                                 794.0, 786.5, 779.0, 770.5, 762.0, 754.0, 745.0, 735.5, 726.0, 716.5,
                                                 707.5, 697.0, 687.0, 677.5, 666.5, 655.0, 645.5, 634.0, 623.0, 612.0,
                                                 600.0, 589.0, 578.0, 566.0, 554.0, 542.0, 529.5, 517.0, 504.5, 492.0,
                                                 479.0, 466.0, 454.0, 440.5, 427.5, 414.5, 402.0, 389.0, 375.0, 362.0,
                                                 349.0, 335.0, 322.0, 310.0, 297.0, 284.0, 271.0, 258.0, 245.0, 232.0,
                                                 223.0, 210.0, 197.0, 188.0, 177.0, 166.0, 155.0, 146.5, 137.0, 130.0, 122.0};
    */
    /*
    QPointF first;
    QPointF second;
    painter->setPen(thinPen);

    for(int i=0; i < 91; i++)
    {
        second.rx() = angleOfTransmissionR[i]*cos(i*_PI/180.0);
        // 448.0 is the distance from the center to Z = 0 + j0
        second.rx() -= 448.0;
        second.ry() = -angleOfTransmissionR[i]*sin(i*_PI/180.0);

        double R = sqrt((second.x()*second.x() + 448.0) + second.y()*second.y());

        first.rx() = second.x() - (second.x() * 7.0 / R);
        first.ry() = second.y() - (second.y() * 7.0 / R);

        // Draw the upper ticks
        painter->drawLine(first, second);

        // Draw the lower ticks
        first.ry() *= -1;
        second.ry() *= -1;
        painter->drawLine(first, second);

    }*/


    // The resistance or conductance component
    painter->setPen(thickPen);
    painter->drawLine(448, 0, -448, 0);

    // 0.0 & 1.0 impedance scale
    painter->save();
    painter->setPen(textPen);
    QFont impFont = painter->font() ;
    impFont.setPointSizeF(impFont.pointSizeF() / 1.2);
    painter->setFont(impFont);
    painter->rotate(-90);
    if (m_chartImpedance<10.0)
        painter->drawText(-5, 0, 30 ,15, Qt::AlignCenter, QString::number(m_chartImpedance,'f',1));
    else
        painter->drawText(-2, 0, 30 ,15, Qt::AlignCenter, QString::number(m_chartImpedance,'f',1));
    painter->setPen(QPen(Qt::black, 3.5, Qt::SolidLine, Qt::RoundCap));
    painter->drawPoint(0, 0);

    painter->drawText(QPointF(2.0,-438.0), QString::number(0.0,'f',1));

    painter->restore();


    // We draw the quality circles if activated
    if (m_showQualityCircles) {
        QPen QCirclesPen(Qt::darkGray, 1.3, Qt::SolidLine, Qt::RoundCap);
        painter->setPen(QCirclesPen);
        for (int i=-1 ; i<=2 ; i+=2)
        {
            painter->drawText(-20, i*-117 , 40, i*20, Qt::AlignCenter, "Q=0.5" );
            drawArc3points(painter, ZtoXYPainterCalc(complexF(0.002 , i*0.001 )), // 0.5 Qcircle
                                    ZtoXYPainterCalc(complexF(0.3, i*0.15)),
                                    ZtoXYPainterCalc(complexF(0.82, i*0.41))  );
            drawArc3points(painter, ZtoXYPainterCalc(complexF(0.98 , i*0.49 )), // 0.5 Qcircle
                                    ZtoXYPainterCalc(complexF(4.0, i*2.0)),
                                    ZtoXYPainterCalc(complexF(1000.0, i*1000.0))  );
            painter->drawText(-20, i*-196 , 40, i*20, Qt::AlignCenter, "Q=1.0" );
            drawArc3points(painter, ZtoXYPainterCalc(complexF(0.003 , i*0.003 )), // 1.0 Qcircle
                                    ZtoXYPainterCalc(complexF(0.3, i*0.3)),
                                    ZtoXYPainterCalc(complexF(0.66, i*0.66))  );
            drawArc3points(painter, ZtoXYPainterCalc(complexF(0.76 , i*0.76 )), // 1.0 Qcircle
                                    ZtoXYPainterCalc(complexF(2.0, i*2.0)),
                                    ZtoXYPainterCalc(complexF(1000.0, i*1000.0))  );
            painter->drawText(-20, i*-288 , 40, i*20, Qt::AlignCenter, "Q=2.0" );
            drawArc3points(painter, ZtoXYPainterCalc(complexF(0.003 , i*0.006 )), // 2.0 Qcircle
                                    ZtoXYPainterCalc(complexF(0.2, i*0.4)),
                                    ZtoXYPainterCalc(complexF(0.42, i*0.84))  );
            drawArc3points(painter, ZtoXYPainterCalc(complexF(0.475 , i*0.95 )), // 2.0 Qcircle
                                    ZtoXYPainterCalc(complexF(1.0, i*2.0)),
                                    ZtoXYPainterCalc(complexF(1000.0, i*1000.0))  );
            painter->drawText(-20, i*-377 , 40, i*20, Qt::AlignCenter, "Q=5.0" );
            drawArc3points(painter, ZtoXYPainterCalc(complexF(0.003 , i*0.015 )), // 5.0 Qcircle
                                    ZtoXYPainterCalc(complexF(0.1, i*0.5)),
                                    ZtoXYPainterCalc(complexF(0.186, i*0.186*5.0))  );
            drawArc3points(painter, ZtoXYPainterCalc(complexF(0.208 , i*0.208*5.0 )), // 5.0 Qcircle
                                    ZtoXYPainterCalc(complexF(0.4, i*0.4*5.0)),
                                    ZtoXYPainterCalc(complexF(1000.0, i*1000.0))  );
            painter->drawText(-20, i*-416 , 40, i*20, Qt::AlignCenter, "Q=10.0" );
            drawArc3points(painter, ZtoXYPainterCalc(complexF(0.003 , i*0.030 )), // 10.0 Qcircle
                                    ZtoXYPainterCalc(complexF(0.05, i*0.5)),
                                    ZtoXYPainterCalc(complexF(0.094, i*0.94))  );
            drawArc3points(painter, ZtoXYPainterCalc(complexF(0.105 , i*1.05 )), // 10.0 Qcircle
                                    ZtoXYPainterCalc(complexF(0.2, i*2.0)),
                                    ZtoXYPainterCalc(complexF(1000.0, i*1000.0))  );
        }
    }

    // Finally we draw all the inner arcs of the Smith chart
    painter->strokePath(thickArcsPath, thickPen);
    painter->strokePath(thinArcsPath, thinPen);
}


void SmithChart::drawDataPoints(QPainter * painter)
{
    //qDebug() << "drawDataPoints()";

    // Draw the data points on the chart
    BezierCurve bezierCurve;  // to draw the bezier curve
    CurvePointsList curvePoints;

    /*QElapsedTimer timerElaps;
    qint64 ns_elaps;
    timerElaps.start();*/

    for (int numStartPoint=0 ; numStartPoint<startImpedancesPoints.size() ; numStartPoint++) // each start points
    {
        for (int n=0 ; n<r_circuit.getSize() ; n++) // each components
        {
            curvePoints.clear();

            QColor tmpPenColor;
            if (m_onlyEndPointsModeEnable) {
                tmpPenColor = Qt::gray;
            }
            else {
                if (r_circuit.getComponentMouseOverState(n) ==true)
                    tmpPenColor = QColor(0,0,255,128); // light blue
                else if (r_circuit.getComponentPreviewState(n)==true)
                    tmpPenColor = Qt::black;
                else
                    tmpPenColor = Qt::blue;
            }

            dataPointsPen.setColor(tmpPenColor);
            lineDataPen.setColor(tmpPenColor);

            QVector< QVector<complexF> > tmpPointsList = r_circuit.getComponentDataPoints(n);

            for (int i=0 ; i<tmpPointsList[numStartPoint].size() ; i++) // each comp value
            {
                if (i==0) // For the first point of each component
                {
                    if (n==0) // between start point and first point
                        curvePoints.append(ZtoXYPainterCalc(startImpedancesPoints[numStartPoint].point));
                    else if ((n>0)&&(r_circuit.getComponentPreviewState(n)==false))  // between last point of previous component and first point of next
                        curvePoints.append(ZtoXYPainterCalc(r_circuit.getComponentDataPoints(n-1).at(numStartPoint).last()));
                    else if ((n>0)&&(r_circuit.getComponentPreviewState(n)==true)) // between last point not in preview and first point of all the preview component
                        curvePoints.append(ZtoXYPainterCalc(r_circuit.getLastPointNotInPreview(numStartPoint, startImpedancesPoints[numStartPoint].point)));
                }

                // All others points : just add the point to curve
                curvePoints.append(ZtoXYPainterCalc(tmpPointsList[numStartPoint][i]));
                /*dataPointsPen.setWidthF(1.0); // 2.0
                painter->setPen(dataPointsPen);
                painter->drawPoint(ZtoXYPainterCalc(tmpPointsList[numStartPoint][i]));*/

                // Last point of each curve in bold
                if (i==tmpPointsList[numStartPoint].size()-1)
                {
                    // Last point of last component (for end points display mode)
                    if ((n==r_circuit.getSize()-1)&&(m_onlyEndPointsModeEnable)) {
                        dataPointsPen.setColor(Qt::red);
                    }
                    dataPointsPen.setWidthF(4.0);
                    painter->setPen(dataPointsPen);
                    painter->drawPoint(ZtoXYPainterCalc(tmpPointsList[numStartPoint][i]));
                }

                // Draw component type under the middle point
                if (r_circuit.getComponentPreviewState(n)==false) // only on components not in preview
                {
                    // Calculate where is the middle point among all the data points for this component
                    int nbPointsComp = tmpPointsList[numStartPoint].size();
                    int middlePoint = (int)(nbPointsComp/2)-1;

                    if ((nbPointsComp>=2)&&(i==middlePoint+1)&&(i>=1))
                    {
                        complexF tmpSub = tmpPointsList[numStartPoint][i] - tmpPointsList[numStartPoint][i-1];
                        QPointF pictTranslate;

                        if (qAbs(tmpSub.real()) > qAbs(tmpSub.imag())) // horizontal line
                            pictTranslate = QPointF(-15.0,+5.0);
                        else  // vertical line
                            pictTranslate = QPointF(-35.0,-15.0);

                        QPixmap pixmapTmp;
                        int sizePix = (zoomScale>=8.0 ? 64 : 32);
                        pixmapTmp = r_circuit.getPixmapComponent(r_circuit.getComponentType(n), sizePix, sizePix);

                        int sizeRectPix = (int)(16.0+16.0/zoomScale);
                        QRectF rectFpict(ZtoXYPainterCalc(tmpPointsList[numStartPoint][i])+pictTranslate, QSizeF(sizeRectPix,sizeRectPix));
                        painter->drawPixmap(rectFpict.toRect(), pixmapTmp);
                    }
                } // end drawing of comp type icon
            } // end for each comp value

            // Lines or curves between each points
            QPainterPath myPath;
            painter->setPen(lineDataPen);
            if (m_bezierCurvesMode==true) { // bezier curves mode : calculate and display a curve between each data point
                // Calculate the bezier curve between each points
                if (curvePoints.size() >= 2)
                {
                    bezierCurve.calcCurveControlPoints(curvePoints);
                    // Draw the path
                    myPath.moveTo(curvePoints[0]);
                    for (int k=0 ; k<curvePoints.size()-1 ; k++)
                        myPath.cubicTo(bezierCurve.getFirstControlPoint(k), bezierCurve.getSecondControlPoint(k), curvePoints[k+1]);
                    painter->drawPath(myPath);
                    //qDebug() << "Draw path";
                }
            }
            else { // no bezier curves mode : just display a line between each data point
                myPath.moveTo(curvePoints[0]);
                for (int k=0 ; k<curvePoints.size() ; k++)
                    myPath.lineTo(curvePoints[k]);
                painter->drawPath(myPath);
            }
        } // end for each component
    }// end for each start point

    //ns_elaps = timerElaps.nsecsElapsed();
    //qDebug() << "Draw points time :" << (double)ns_elaps/(double)1000.0 << "us";

    // Start points drawing (after the data points, so the drawing will be above them)
    for (int numStartPoint=0 ; numStartPoint<startImpedancesPoints.size() ; numStartPoint++) // each start points
    {
        //zoomScale>=4.0 ? dataPointsPen.setWidthF(5.0) : dataPointsPen.setWidthF(10.0);
        dataPointsPen.setColor(Qt::blue);
        dataPointsPen.setWidthF(4.0+6.0/zoomScale);
        painter->setPen(dataPointsPen);
        painter->drawPoint(ZtoXYPainterCalc(startImpedancesPoints[numStartPoint].point));

        painter->setPen(textPen);
        painter->setFont(QFont("monospace", 6.0+6.0/zoomScale, QFont::Light));
        painter->drawText(ZtoXYPainterCalc(startImpedancesPoints[numStartPoint].point)+QPointF(-9.0-9.0/zoomScale,10.0+12.0/zoomScale), QString("Start")+QString::number(numStartPoint));
    }

    // Draw the free points on the chart
    if (m_showTargetPoints)
    {
        freePointsPen.setWidthF(4.0+6.0/zoomScale);
        painter->setPen(freePointsPen);

        painter->drawPoint(ZtoXYPainterCalc(targetImpedancePoint));
        painter->setPen(textPen);
        painter->setFont(QFont("monospace", 6.0+6.0/zoomScale, QFont::Light));
        painter->drawText(ZtoXYPainterCalc(targetImpedancePoint)+QPointF(-11.0-11.0/zoomScale,10.0+12.0/zoomScale), "Target");
    }

    // Draw the last path between each end points in sweep mode
    if (m_onlyEndPointsModeEnable)
    {
        curvePoints.clear();
        for (int numStartPoint=0 ; numStartPoint<startImpedancesPoints.size() ; numStartPoint++) { // each start points
            if (r_circuit.getSize()>0)
                curvePoints.append(ZtoXYPainterCalc(r_circuit.getComponentDataPoints(r_circuit.getSize()-1).at(numStartPoint).last()));
        }
        // Calculate the bezier curve between each points
        if (curvePoints.size() >= 2)
        {
            lineDataPen.setWidthF(2.0);
            lineDataPen.setColor(Qt::red);
            painter->setPen(lineDataPen);
            bezierCurve.calcCurveControlPoints(curvePoints);
            // Draw the path
            QPainterPath myPath;
            myPath.moveTo(curvePoints[0]);
            for (int k=0 ; k<curvePoints.size()-1 ; k++)
                myPath.cubicTo(bezierCurve.getFirstControlPoint(k), bezierCurve.getSecondControlPoint(k), curvePoints[k+1]);
            painter->drawPath(myPath);
            //qDebug() << "Draw path";
        }
    }

    //qDebug() << "End of drawDataPoints()";
}


void SmithChart::drawAddCompPixmap(QPainter * painter)
{
    // Find the last point displayed on screen (start point or last component data point if any)
    complexF lastPoint = r_circuit.getLastPointNotInPreview(0, startImpedancesPoints[0].point);
    m_suppPictPos_p = ZtoXYPainterCalc(lastPoint) + QPointF(20.0,-30.0); // '+' pix on the last simulation point + (20;-30)

    m_otherCompPosPicts_p[CAPA_S]=     m_suppPictPos_p+polarToCartXY(m_radiusAnimation, 192+40*0);
    m_otherCompPosPicts_p[SELF_S]=     m_suppPictPos_p+polarToCartXY(m_radiusAnimation, 192+40*1);
    m_otherCompPosPicts_p[RESIST_S]=   m_suppPictPos_p+polarToCartXY(m_radiusAnimation, 192+40*2);
    m_otherCompPosPicts_p[LINE_S]=     m_suppPictPos_p+polarToCartXY(m_radiusAnimation, 192+40*3);
    m_otherCompPosPicts_p[CAPA_P]=     m_suppPictPos_p+polarToCartXY(m_radiusAnimation, 192+40*4);
    m_otherCompPosPicts_p[SELF_P]=     m_suppPictPos_p+polarToCartXY(m_radiusAnimation, 192+40*5-5);
    m_otherCompPosPicts_p[RESIST_P]=   m_suppPictPos_p+polarToCartXY(m_radiusAnimation, 192+40*6);
    m_otherCompPosPicts_p[LINE_OPEN]=  m_suppPictPos_p+polarToCartXY(m_radiusAnimation, 192+40*7);
    m_otherCompPosPicts_p[LINE_SHORT]= m_suppPictPos_p+polarToCartXY(m_radiusAnimation, 192+40*8);

    if ((m_compPictPrevUnderMouse==NOT_DEF) || (m_showOtherCompPicts==false))
        painter->setOpacity(1.0);
    else
        painter->setOpacity(0.2);
    QPixmap pixmapTmp = r_circuit.getPixmapSuppComp(32*zoomScale, 32*zoomScale);
    double sizePict = 20.0+17.0/zoomScale;
    QRectF rectFpict(m_suppPictPos_p-QPointF(sizePict/2.0, sizePict/2.0), QSizeF(sizePict, sizePict));
    painter->drawPixmap(rectFpict, pixmapTmp, pixmapTmp.rect());
    if (m_mouseOverSuppPixmap==true) // if mouse on '+' icon, draw a "gradient style" border around the icon
    {
        painter->setPen(QPen(Qt::darkGray, 1.0, Qt::SolidLine, Qt::RoundCap)); // grey border
        painter->setOpacity(0.8);
        painter->drawRect(QRectF(m_suppPictPos_p-QPointF(sizePict/2.0, sizePict/2.0)+QPointF(-0.4,-0.4), QSizeF(sizePict, sizePict)+QSizeF(0.8,0.8)));
        painter->setOpacity(0.5);
        painter->drawRect(QRectF(m_suppPictPos_p-QPointF(sizePict/2.0, sizePict/2.0)+QPointF(-1.0,-1.0), QSizeF(sizePict, sizePict)+QSizeF(2.0,2.0)));
        painter->setOpacity(0.2);
        painter->drawRect(QRectF(m_suppPictPos_p-QPointF(sizePict/2.0, sizePict/2.0)+QPointF(-1.6,-1.6), QSizeF(sizePict, sizePict)+QSizeF(3.2,3.2)));
    }
    painter->setOpacity(1.0);

    // if user mouse is on this supp comp picture, show the others components
    if (m_showOtherCompPicts)
    {
        for (int k=0 ; k<NB_COMP_TYPE-1 ; k++) // each component type
        {
            if ((m_compPictPrevUnderMouse!=k)&&(m_compPictPrevUnderMouse!=NOT_DEF))
                painter->setOpacity(0.2);
            pixmapTmp = r_circuit.getPixmapComponent((ComponentType)k,32*zoomScale,32*zoomScale);
            rectFpict = QRectF(m_otherCompPosPicts_p[k]-QPointF(sizePict/2.0, sizePict/2.0), QSizeF(20.0+17.0/zoomScale,20.0+17.0/zoomScale));
            painter->drawPixmap(rectFpict, pixmapTmp, pixmapTmp.rect());
            painter->setOpacity(1.0);
        }
    }
}



void SmithChart::radiusAnimatorInternal(QVariant val)
{
    m_radiusAnimation = val.toInt();
    if ((QVariant)m_radiusAnimation >= m_animIconRadius->endValue())
        m_radiusAnimationRunning=false;
    update();
}

void SmithChart::textOpacityAnimatorFct(QVariant val)
{
    m_helpTextOpacity = val.toFloat();
    //qDebug() << "Animtor : " << m_helpTextOpacity;
    //if ((QVariant)m_helpTextOpacity >= m_animTextOpacity->endValue())
    //    m_textOpacityAnimationRunning=false;
    update();
}

void SmithChart::endPointsModeEnable(bool state)
{
    m_onlyEndPointsModeEnable = state;
    update();
}


ComponentType SmithChart::findIconUnderMouse(QPointF posMouse_p)
{
    int sizePixDiv2 = (int)((20.0+17.0/zoomScale)/2.0);
    for (int i=0 ; i<NB_COMP_TYPE ; i++)
    {
         // mouse in one of the icons around '+' pixmap
        if ((abs(posMouse_p.x()-m_otherCompPosPicts_p[i].x())<sizePixDiv2) && (abs(posMouse_p.y()-m_otherCompPosPicts_p[i].y())<sizePixDiv2))
        {
            return ((ComponentType)i);
        }
    }
    return NOT_DEF;
}

int SmithChart::findDataPointUnderMouse(QPointF posMouse_w, float distanceMax)
{
    // Check for each data points distance
    int numClosest=NONE;
    float minDist;
    float tmpDist;

    for (int i=0 ; i<r_circuit.getSize() ; i++) // each components
    {
        const QVector< QVector<complexF> > &tmpPoints = r_circuit.getComponentDataPoints(i);

        for (int j=0 ; j<tmpPoints.size() ; j++) // each start points
        {
            for (int k=0 ; k<tmpPoints[j].size() ; k++) // each freqs
            {
                tmpDist = vectorLength(painterToWidgetTransform(ZtoXYPainterCalc(tmpPoints[j][k])), posMouse_w);
                if (tmpDist < distanceMax) {
                    if ((numClosest==NONE) || (tmpDist < minDist))
                    {
                        minDist=tmpDist;
                        numClosest=i;
                        //qDebug() << "Scroll near component n°" << componentLinkWithPoint[0][k] << "=> " << r_circuit.getComponentName(componentLinkWithPoint[0][k]);
                    }
                }
            }
        }
    }
    return numClosest;

}

int SmithChart::findStartPointUnderMouse(QPointF posMouse_w)
{
    int numClosest=NONE;
    float tmpDist;
    float minDist;

    // Check for each start points
    for (int i=0 ; i<startImpedancesPoints.size() ; i++) // each start points
    {
        tmpDist=(painterToWidgetTransform(ZtoXYPainterCalc(startImpedancesPoints[i].point))-posMouse_w).manhattanLength();
        //qDebug() << tmpLength;
        if (tmpDist < (20+zoomScale*2) ) { // if mouse is close of a start point
            if ((numClosest==NONE) || (tmpDist<minDist)) {
                numClosest=i;
                minDist=tmpDist;
            }
        }
    }
    return numClosest;
}
bool SmithChart::findTargetPointUnderMouse(QPointF posMouse_w)
{
    //QPointF posMouse_p = widgetToPainterTransform(posMouse_w);
    qreal tmpLength;

    // Check for target point
    tmpLength=(painterToWidgetTransform(ZtoXYPainterCalc(targetImpedancePoint))-posMouse_w).manhattanLength();
    if (tmpLength < (20+zoomScale*2) ) { // if mouse is close of the target point
        return true;
    }
    return false;
}



double SmithChart::vectorLength(QPointF point1, QPointF point2)
{
    double x_sub = point2.x() - point1.x();
    double y_sub = point2.y() - point1.y();
    double ret = sqrt( x_sub*x_sub+y_sub*y_sub );

    return ret;
}


bool SmithChart::getPointMovingState()
{
    return m_startPointSelected;
}


void SmithChart::calculateSmithChartInsideArcs()
{
    /*
    Inside constant Z arcs
    */
//    qDebug()<<ZtoXYPainterCalc(complexF(thinLinesConstantZ[0],thinLinesConstantZ[1]));
//    qDebug()<<ZtoXYPainterCalc(complexF(thinLinesConstantZ[0],thinLinesConstantZ[2]));
//    qDebug()<<ZtoXYPainterCalc(complexF(thinLinesConstantZ[0],thinLinesConstantZ[3]));

    for (int i=0 ; i<NB_THIN_LINES_CST_Z ; i++) {
        addArc3pointsToPath(thinPen, ZtoXYPainterCalc(complexF(thinLinesConstantZ[0+i*4], thinLinesConstantZ[1+i*4])),
                                ZtoXYPainterCalc(complexF(thinLinesConstantZ[0+i*4], thinLinesConstantZ[2+i*4])),
                                ZtoXYPainterCalc(complexF(thinLinesConstantZ[0+i*4], thinLinesConstantZ[3+i*4])));
    }

    for (int i=0 ; i<NB_THICK_LINES_CST_Z ; i++) {
        addArc3pointsToPath(thickPen, ZtoXYPainterCalc(complexF(thickLinesConstantZ[0+i*4], thickLinesConstantZ[1+i*4])),
                                ZtoXYPainterCalc(complexF(thickLinesConstantZ[0+i*4], thickLinesConstantZ[2+i*4])),
                                ZtoXYPainterCalc(complexF(thickLinesConstantZ[0+i*4], thickLinesConstantZ[3+i*4])));
    }

    /*
    Inside constant Ro arcs
    */
    for (int i=0 ; i<NB_THIN_LINES_CST_Ro ; i++) {
        // Upper thin arcs
        addArc3pointsToPath(thinPen, ZtoXYPainterCalc(complexF(thinLinesConstantRo[1+i*4], thinLinesConstantRo[0+i*4])),
                                ZtoXYPainterCalc(complexF(thinLinesConstantRo[2+i*4], thinLinesConstantRo[0+i*4])),
                                ZtoXYPainterCalc(complexF(thinLinesConstantRo[3+i*4], thinLinesConstantRo[0+i*4])));

        // Lower thin arcs
        if (i>=10) { // don(t know why, but Qt don't draw well circle arc on bottom side with too much radius, so the 10 first have to be translated a bit
            addArc3pointsToPath(thinPen, ZtoXYPainterCalc(complexF(thinLinesConstantRo[1+i*4], -thinLinesConstantRo[0+i*4])),
                                    ZtoXYPainterCalc(complexF(thinLinesConstantRo[2+i*4], -thinLinesConstantRo[0+i*4])),
                                    ZtoXYPainterCalc(complexF(thinLinesConstantRo[3+i*4], -thinLinesConstantRo[0+i*4])));
        }
        else
            addArc3pointsToPath(thinPen, ZtoXYPainterCalc(complexF(thinLinesConstantRo[1+i*4]+(10-i)*0.0005, -thinLinesConstantRo[0+i*4])),
                                    ZtoXYPainterCalc(complexF(thinLinesConstantRo[2+i*4], -thinLinesConstantRo[0+i*4])),
                                    ZtoXYPainterCalc(complexF(thinLinesConstantRo[3+i*4]+(10-i)*0.0005, -thinLinesConstantRo[0+i*4])));
    }

    for (int i=0 ; i<NB_THICK_LINES_CST_Ro ; i++) {
        // Upper thick arc
        addArc3pointsToPath(thickPen, ZtoXYPainterCalc(complexF(thickLinesConstantRo[1+i*4], thickLinesConstantRo[0+i*4])),
                                ZtoXYPainterCalc(complexF(thickLinesConstantRo[2+i*4], thickLinesConstantRo[0+i*4])),
                                ZtoXYPainterCalc(complexF(thickLinesConstantRo[3+i*4], thickLinesConstantRo[0+i*4])));
        // Lower thick arc
        if (i!=0)
            addArc3pointsToPath(thickPen, ZtoXYPainterCalc(complexF(thickLinesConstantRo[1+i*4], -thickLinesConstantRo[0+i*4])),
                                    ZtoXYPainterCalc(complexF(thickLinesConstantRo[2+i*4], -thickLinesConstantRo[0+i*4])),
                                    ZtoXYPainterCalc(complexF(thickLinesConstantRo[3+i*4], -thickLinesConstantRo[0+i*4])));
        else
            addArc3pointsToPath(thickPen, ZtoXYPainterCalc(complexF(thickLinesConstantRo[1+i*4]+0.003, -thickLinesConstantRo[0+i*4])),
                                    ZtoXYPainterCalc(complexF(thickLinesConstantRo[2+i*4], -thickLinesConstantRo[0+i*4])),
                                    ZtoXYPainterCalc(complexF(thickLinesConstantRo[3+i*4]+0.003, -thickLinesConstantRo[0+i*4])));
    }

}



