/****************************************************************************
 * QSmithTool :                                                             *
 *   Copyright (C) 2015 by Sébastien Lissarre                               *
 *   sebastien.lissarre@gmail.com                                           *
 *                                                                          *
 * Smith Chart drawing base :                                               *
 *   Copyright (C) 2008 by Lisandro Damián Nicanor Pérez Meyer              *
 *   perezmeyer@gmail.com                                                   *
 *                                                                          *
 *                                                                          *
 *  This file is part of QSmithTool.                                        *
 *                                                                          *
 *  QSmithTool is free software: you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation, either version 3 of the License, or       *
 *  (at your option) any later version.                                     *
 *                                                                          *
 *  QSmithTool is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with QSmithTool.  If not, see <http://www.gnu.org/licenses/>.     *
 ***************************************************************************/


#ifndef SMITHCHART_H
#define SMITHCHART_H

#include <QWidget>
#include <QPen>
#include <QPainterPath>
#include <QPainter>
#include <QRectF>
#include <QVector>
#include <QPoint>
#include <QPointF>
#include <QFont>
#include <math.h>
#include <QMouseEvent>
#include <QDebug>
#include <QTransform>
#include <QApplication>
#include <QInputDialog>
#include <QPropertyAnimation>
#include <QElapsedTimer>
#include <QFileDialog>

#include <cmath>

#include "defines.h"
#include "chartLinesList.h"
#include "circuit.h"
#include "dialogParametersDiscrete.h"
#include "dialogParametersLine.h"
#include "rfFctsLibrary.h"
#include "bezierCurves.h"


// Forward declarations
class QPaintEvent;
class QPainterPath;
class QPoint;

/// Smith chart graphic class
/**
Shows a Smith chart and is able to draw data points into it.
*/

class SmithChart : public QWidget
{
	Q_OBJECT
public:
    SmithChart(double chartImpedance, Circuit &_circuit);
	~SmithChart();

    /**
    Add the data to be drawn.
    This function takes impedances already normalized.
    */
    void addPoint(const QPointF Z, int compNumberInlist);
    void addPoints(const QVector<complexFAndFreq> ZVector, int compNumberInlist);

    /// Modify start points (not normalized here!!)
    void setStartPoints(const QVector<complexFAndFreq> _ZVector);
    void setStartPointsNormalized(const QVector<complexFAndFreq> _ZVectorNorm);
    void moveStartPoint(int _numStartPoint, const complexF _newPoint);
    /// Return the start points normalized (50ohm = 1.0+j0.0)
    QVector<complexFAndFreq> & getStartPointsNorm();
    complexF getFirstStartPointDenorm();

    /// Read/Modify target point
    complexF getTargetPoint();
    void setTargetPoint(const complexF _newPoint);
    void setTargetPointNormalized(const complexF _newPoint);

    /// Enable/Disable target points on the chart
    void enableTargetPoints(bool _enable);

    /// Enable/Disable the qcircles drawing
    void enableQCirclesDrawing(bool _enable);

    /// Enable/disable bezier cruves drawing
    void enableBezierCurvesDrawing(bool _enable);

    /// Lock for start points & target point
    bool getStartPointsLockState() { return m_startPointsLocked;}
    void setStartPointsLockState(bool state) { m_startPointsLocked = state;}
    bool getTargetPointLockState() { return m_targetPointLocked;}
    void setTargetPointLockState(bool state){ m_targetPointLocked = state;}

    /// Return the chart Zo impedance (50ohm by default)
    double getChartImpedance();
    /// Set the chart Zo impedance (50ohm by default)
    void setChartImpedance(double impedance);

    /// Return the impedance under the user mouse
    complexF getLastZPosMouse();
    QPointF getLastXYPosMouse();

    /// Impedance in chart to coordinates XY conversion (to painter coord)
    static QPointF ZtoXYPainterCalc(const complexF &pointZ);
    /// XY coord => normalized chart impedance conversion
    complexF XYWidgetToZcalc(const QPointF &pointXY_w);

    /// Translate the QWidget coord to the painter coord
    QPointF widgetToPainterTransform(QPointF point);
    QRectF widgetToPainterTransform(QRectF rect);
    /// Translate the painter coord (-512;-512;1024;1024) to the QWidget coord (decided when the window is resized...)
    QPointF painterToWidgetTransform(QPointF point);
    QRectF painterToWidgetTransform(QRectF rect);

    /// Return the length between two points
    double vectorLength(QPointF point1, QPointF point2);

    /// Return true if the user is moving a point
    bool getPointMovingState();

    /// Called by doubleClickEvent in chart or in component list (mainwindow) to modify parameters
    void componentDoubleClick(int numComponent);

    /// KeypressEvent
    void keyPressEvent( QKeyEvent *event );

public slots:
    /// Clear the data and reinit data vectors
    void clearStartImpedancePoints();
    void clearTargetImpedancePoint();

    /// Set all the lines permitivity
    void setPermitivityToAllLines(double relPerm);

    /// Enable/disable sweep mode (only end points showed in red)
    void endPointsModeEnable(bool state);

private slots:
    void radiusAnimatorInternal(QVariant val);
    void textOpacityAnimatorFct(QVariant val);

signals:
    void posUnderMouseUpdated();
    void startSim(SimQuality _simQuality);
    void componentsListUpdated();
    void startPointsUpdated();
    void targetPointUpdated();
    void autoSaveSignal();
    void undoActionSignal();

protected:
	void paintEvent(QPaintEvent *event);
    void resizeEvent(QResizeEvent *);

    void keyReleaseEvent( QKeyEvent *event );
    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);
    void mouseDoubleClickEvent( QMouseEvent *event );
    void wheelEvent ( QWheelEvent * event );


private:
    /// For each new drawing of the Smith Chart (started by paintEvent) :
    /// The chart background (lines & ...)
    void drawChart(QPainter * painter);
    void generatePixmapChartBackground();
    /// The data points (blue, green, red,...)
    void drawDataPoints(QPainter * painter);
    /// Draw the '+' pixmap wich allow to add a new component
    void drawAddCompPixmap(QPainter * painter);

	/// Calculates the inside arcs and loads it to the painter paths
    void calculateSmithChartInsideArcs();

    /// Recalculate the zoom windows coordinates for next zoom
    void updateZoomRectangle(QPoint posMouse, bool zoomInOut);

    /// Find the component icon under the moouse (the closest)
    ComponentType findIconUnderMouse(QPointF posMouse_p );
    /// Find the data point under mouse
    int findDataPointUnderMouse(QPointF posMouse_w, float distance);
    /// Find if mouse is over a start point
    int findStartPointUnderMouse(QPointF posMouse_p);
    /// Find if mouse is over a target point
    bool findTargetPointUnderMouse(QPointF posMouse_w);

    /// Call arc3pointsCalc to calculate the arc and add it to the strokePath QPainterPath
    void addArc3pointsToPath(QPen pen, QPointF a, QPointF b, QPointF c);
    /// Call arc3pointsCalc to calculate the arc and paint it
    void drawArc3points(QPainter *painter, QPointF a, QPointF b, QPointF c);
    /// Take 3 points a,b,c and store the _resRect,_resStartAngle,_resSweepAngle (in degree) for the arc passing through a,b,c
    void arc3pointsCalc(QPointF a, QPointF b, QPointF c, QRectF *_resRect, double *_resStartAngle, double *_resSweepAngle);
    /// Calculate angle in degree between to points
    double angleBetween(QPointF pt1, QPointF pt2);

    /// The complex data vector containing the start points, in normalized value (50ohm = 1.0+0.0j)
    QVector< complexFAndFreq > startImpedancesPoints;

    /// For some user target points on the Smith Chart
    complexF targetImpedancePoint;

    /// Reference to the circuit
    Circuit &r_circuit;

    /// Smith chart default Zo
    double m_chartImpedance;

    /// Last impedance under the mouse
    complexF posZUnderMouse;
    /// Last XY position under the mouse
    QPointF posXYUnderMouse_w; // in widget coord

    /// Variables to store zoom/key pressed state
    double zoomScale;
    QPointF zoomTranslate;
    QRectF newZoomRect_p; // in painter coord
    QPointF newZoomRectCenter_p; // in painter coord
    bool m_zoomButtonPressed;
    bool m_controlKeyPressed;
    int m_startPointSelected;
    bool m_targetPointSelected;

    /// Enable/disable sweep mode (only end points showed in red)
    bool m_onlyEndPointsModeEnable;

    /// The position of the "supp comp (+)" icon and the component icon on circle around
    QPointF m_suppPictPos_p; // in painter coord
    QPointF m_otherCompPosPicts_p[NB_COMP_TYPE]; // in painter coord
    bool m_showOtherCompPicts;
    ComponentType m_compPictPrevUnderMouse;
    bool m_mouseOverSuppPixmap;

    /// To animate the pixmap around the '+' pict
    QVariantAnimation *m_animIconRadius;
    int m_radiusAnimation;
    bool m_radiusAnimationRunning;
    bool m_hideKeyPressed;

    QVariantAnimation *m_textOpacityAnimator;
    double m_helpTextOpacity;
    bool m_textOpacityAnimationRunning;

	/// Show the interpolated line between the data points
    bool m_showTargetPoints;

    /// Show or not the QCircles
    bool m_showQualityCircles;

    /// Lock for start points & target point
    bool m_startPointsLocked;
    bool m_targetPointLocked;

    /// Qpainter to draw the chart
    QPainter *painter;
    bool m_painterInitialized;
	/// Pen for the text
    QPen textPen;
	/// Thick pen
	QPen thickPen;
	/// Thin pen
	QPen thinPen;
	/// Pen for the data points
    QPen dataPointsPen;
    /// Pen for the free points
    QPen freePointsPen;
	/// Pen for the lines interpolating the data points
	QPen lineDataPen;

	/// Path for the thin arcs
    QPainterPath thinArcsPath;
	/// Path for the thick arcs
    QPainterPath thickArcsPath;

    /// Contain the background pixmap
    QPixmap pixmapChartBackground;
    bool pixmapChartBackgroundGenerated;

    /// Dialog to modify components parameters
    dialogParametersLine *dialogLine;
    dialogParametersDiscrete *dialogComp;

    /// If true, bezier curves are calculated and drawed between data points (slow the drawing)
    bool m_bezierCurvesMode;

};

#endif //SIMULATION_H

